using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace test1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            dplot.DPlot_FindDPLOTLIB();
            dplot.DPlot_SetErrorMethod((2));
        }

        private void button1_Click(object sender, EventArgs e)
        {
            int DocNum;
            int i;
            int NP;
            double[] x;
            double[] y;
            string cmds;

            /* Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
            In this case we've used DATA_XYYY (one X array for one or more Y arrays).
			
            Since the X values are evenly spaced we could also use
            DATA_DXY  - X has only 2 elements, DX and X0
			  
            And you can ALWAYS use
            DATA_XYXY - Each curve uses its own X array. This is the only
            option available if the curves have different X values or
            a different number of points.
            */

            NP = 1001;
            x = new double[NP];
            y = new double[NP * 2];
            for (i = 0; i < NP; i++)
            {
                x[i] = (4.0 * i) / (NP - 1);
                y[i] = Math.Sin(Math.PI * x[i]);
                y[i + NP] = Math.Cos(Math.PI * x[i]);
            }

            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_XYYY;
            dp.MaxCurves = 2;           // Must be >= number of curves we plot 
            dp.MaxPoints = NP;          // Anything >= NP will do 
            dp.NumCurves = 2;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.LegendX = 0.05F;
            dp.LegendY = 0.05F;
            dp.NP[0] = NP;
            dp.NP[1] = NP;
            dp.LineType[0] = dplot.LINESTYLE_SOLID;
            dp.LineType[1] = dplot.LINESTYLE_LONGDASH;
            dp.Title1 = "Data sent to DPlot via dplotlib64.dll";
            dp.XAxis = "x";
            dp.YAxis = "y";

            /* The command string can be as complex as you like, limited to 32K characters.
            As an alternative, if your program will be producing many similar plots then
            you might prefer to create a preferences file, edit the file with a text 
            editor to remove unwanted entries, and use the [GetPreferences("filename")]
            command. 
            */
            cmds = "[ManualScale(0,-1.25,4,1.25)][TickInterval(1,0.5,0.25)]"+
                "[Caption(\"DPLOTLIB XY Plot\")]"+
                "[Legend(1,\"sin({\\sp}x)\")][Legend(2,\"cos({\\sp}x)\")]" +
                "[DocMaximize()][ClearEditFlag()]";
            DocNum = dplot.DPlot_Plot8(ref dp, x, y, cmds);
            if (DocNum > 0)
            {
                StringBuilder peaks = new StringBuilder(512);
                int size = peaks.Capacity;
                int ret;

                ret = dplot.DPlot_Request(DocNum, "ListPeaks", peaks, ref size);
                if (ret > 0)
                    MessageBox.Show(peaks.ToString(), "Peaks", MessageBoxButtons.OK);
            }
        }
        private void button2_Click(object sender, EventArgs e)
        {
            int DocNum;
            int i, j;
			int Nx, Ny;
			double[] extents;
			double xx, yy;
			double[] z;
            string cmds;
			
			/* Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y) */
			
			Nx = 61;	// Columns
			Ny = 41;	// Rows
			z = new double[Nx*Ny];
            extents = new double[4];
			extents[0] = -3.0;		// xlo
			extents[1] = -2.0;		// ylo
			extents[2] =  3.0;		// xhi
			extents[3] =  2.0;		// yhi
			for(i=0; i<Nx; i++)
			{
				xx = extents[0] + ((extents[2]-extents[0])*i)/(Nx-1);
				for(j=0; j<Ny; j++)
				{
					yy = extents[1] + ((extents[3]-extents[1])*j)/(Ny-1);
					z[i*Ny+j] = Math.Sin(xx)*Math.Cos(yy);
				}
			}
            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_3D;
            dp.MaxCurves = Nx-1;        // Horizontal grid size
            dp.MaxPoints = Ny-1;        // Vertical grid size 
            dp.NumCurves = 1;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.Title1 = "Data sent to DPlot via dplotlib64.dll";
            dp.Title2 = "z = sin(x)*cos(y)";
            dp.XAxis = "x";
            dp.YAxis = "y";

			cmds = "[Caption(\"DPLOTLIB Contour Plot / Gridded Data\")]"+
                "[Contour3D(1)][ContourMethod(0)][ContourGrid(0)][ContourAxes(1)][ContourView(30,20)]"+
                "[ContourLevels(20,-1,1)][ContourScales(1,1,1)][ContourLighting(2,0.05)]"+
                "[FontPoints(1,10)][FontPoints(6,10)]"+
                "[FontPoints(4,14)][FontPoints(5,14)][FontPoints(8,14)]"+
                "[ZAxisLabel(\"z\")][Size(0,0,0,0)][DocMaximize()]";
            DocNum = dplot.DPlot_Plot8(ref dp, extents, z, cmds);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            int DocNum;
            int i;
			int	Nz;
			double[] Node;
            string cmds;
            Random rand;
            double[] dummy;
			
			/* Contour plot with randomly spaced points.
			The random feature of this example is a bit nonsensical since we're using 
			an equation to generate the data. This DPlot feature is much more useful 
			for truly random data and/or data values that aren't organized on a 
			rectangular grid. */
            Nz = 10000;
			Node = new double[Nz*3];
            rand = new Random(1);
            dummy = new double[1];

            /* NOTE to any C# gurus: The array arrangement is very inconvenient and a doubly-dimensioned
             * array would be much more clear (and that's what the 32-bit version uses). But it seems
             * the array in the interface for DPlot_Plot8 MUST be declared as "double[] x" rather than
             * "ref double x", or only the first element gets passed correctly. By itself that's not a 
             * problem, but now I can't use double-dimension arrays in those calls. There is probably 
             * some way to do this with an alias, but this isn't really my area of expertise. If anybody
             * knows how to do this with "double[,] Node" then please let me know!
             */
			for( i=0; i < Nz; i++ ) {
                Node[i*3] = -Math.PI + 2 * Math.PI * rand.NextDouble();
                Node[i*3+1] = -Math.PI + 2 * Math.PI * rand.NextDouble();
				Node[i*3+2] = (float)(Math.Sin(Node[i*3]) * Math.Cos(Node[i*3+1]));
			}
            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_3DR;
            dp.MaxCurves = 1;
            dp.MaxPoints = Nz;
            dp.NumCurves = 1;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.Title1 = "Data sent to DPlot via dplotlib64.dll";
            dp.Title2 = "z = sin(x)*cos(y)";
            dp.XAxis = "x";
            dp.YAxis = "y";

			cmds = "[Caption(\"DPLOTLIB Contour Plot / Random Data\")]" +
                "[Contour3D(0)][ContourGrid(0)][ContourMethod(1)][ContourLevels(21,-1,1)]"+
                "[FontPoints(1,10)][FontPoints(6,10)]"+
                "[FontPoints(4,14)][FontPoints(5,14)][DocMaximize()]";
            DocNum = dplot.DPlot_Plot8(ref dp, dummy, Node, cmds);
        }

        private void button4_Click(object sender, EventArgs e)
        {
            /* 3D scatter plot - random points on 3 spheres. */

            int DocNum;
            int i, j;
			int	NC, Nz;
			double zz, radius, theta, phi;
			double theta_inc;
			double xmin, xmax, ymin, ymax, zmin, zmax;
			double xx, yy;
			double[] Node;
            double[] dummy;
            string cmds;
            Random rand;

			Nz = 1000;      // Number of points in each of the NC data sets. Data sets CAN have
							// differing numbers of points. In either case, the Node array is 
							// packed with no unused nodes. For example the first point in the
							// 2nd data set should immediately follow the last point in the 1st
							// data set.
			NC = 3;
            rand = new Random();
            Node = new double[NC*Nz*3];
            dummy = new double[1];
			xmin = xmax = 0.0;
			ymin = ymax = 0.0;
			zmin = zmax = 0.0;
            theta_inc = 2.0*Math.Sqrt((double)Nz);
			for(j=0; j<NC; j++)
			{
				xx = -5.0+10.0*rand.NextDouble();
				yy = -5.0+10.0*rand.NextDouble();
				zz = -5.0+10.0*rand.NextDouble();
				radius = 4.0+4.0*rand.NextDouble();
				xmin = Math.Min(xmin,xx-radius);
				xmax = Math.Max(xmax,xx+radius);
                ymin = Math.Min(ymin, yy - radius);
                ymax = Math.Max(ymax, yy + radius);
                zmin = Math.Min(zmin, zz - radius);
                zmax = Math.Max(zmax, zz + radius);
				for( i=j*Nz; i < (j+1)*Nz; i++ ) {
					phi = -Math.PI/2+Math.PI*(double)(i-j*Nz)/(double)Nz;
					theta = theta_inc*Math.PI*(double)(i-j*Nz)/(double)Nz;
					Node[i*3] = xx + radius*Math.Cos(theta)*Math.Cos(phi);
					Node[i*3+1] = yy + radius*Math.Sin(theta)*Math.Cos(phi);
					Node[i*3+2] = zz + radius*Math.Sin(phi);
				}
			}
            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_3DS;
            dp.MaxCurves = NC;
            dp.MaxPoints = Nz;
            dp.NumCurves = NC;
            dp.NP[0] = dp.NP[1] = dp.NP[2] = Nz;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.Title1 = "Data sent to DPlot via dplotlib64.dll";
            dp.Title2 = "Points on Spheres";
            dp.XAxis = "x";
            dp.YAxis = "y";

            // Force extents to whole numbers; these values used in ManualScale command
            // below.
            xmin = Math.Floor(xmin);
			xmax = Math.Ceiling(xmax);
			ymin = Math.Floor(ymin);
			ymax = Math.Ceiling(ymax);
			zmin = Math.Floor(zmin);
			zmax = Math.Ceiling(zmax);

            cmds = String.Format("[Caption(\"DPLOTLIB 3D Scatter Plot\")]" +
                "[TickInterval(1,1,1,1)][ManualScale({0:N},{1:N},{2:N},{3:N},{4:N},{5:N})]" +
                "[ColorBasis(1)][ContourColorScheme(2)]" +
                "[ContourCustomColors(255,65280,16711680)][ContourScales(1,1,1)]" +
                "[FontPoints(1,10)][FontPoints(6,10)]" +
                "[FontPoints(4,14)][FontPoints(5,14)][FontPoints(8,14)]" +
                "[Legend(1,\"Sphere 1\")][Legend(2,\"Sphere 2\")][Legend(3,\"Sphere 3\")]"+
                "[SymbolOutline(1)][DocMaximize()][ClearEditFlag()]",
                xmin, ymin, xmax, ymax, zmin, zmax);
            DocNum = dplot.DPlot_Plot8(ref dp, dummy, Node, cmds);
        }

        private double DegToRad(double angle)
        {
            return angle * Math.PI / 180;
        }
        private void button5_Click(object sender, EventArgs e)
        {
            // surface plot with concave edges
            int DocNum;
			double[] Star;
            double[] Border;
			double x0, y0, side;
            double[] dummy;
            string cmds;

            x0 = 0;
			y0 = 0;
			side = 10.0;

            Star = new double[10*3];
            dummy = new double[1];
			Star[ 0] = x0 - 0.5*side;
			Star[ 1] = y0 - 0.5*side*Math.Tan(DegToRad(54));
			Star[ 2] = 0.0;
			Star[ 3] = x0;
			Star[ 4] = Star[ 1]+0.5*side*Math.Tan(DegToRad(36));
			Star[ 5] = 1.0;
			Star[ 6] = x0 + 0.5*side;
			Star[ 7] = Star[ 1];
			Star[ 8] = 0.0;
			Star[ 9] = Star[ 6]-0.5*side*(Math.Sin(DegToRad(18))/Math.Cos(DegToRad(36)));
			Star[10] = Star[ 7]+0.5*side*(Math.Cos(DegToRad(18))/Math.Cos(DegToRad(36)));
			Star[11] = 1.0;
			Star[12] = Star[ 6]+side*Math.Sin(DegToRad(18));
			Star[13] = Star[ 7]+side*Math.Cos(DegToRad(18));
			Star[14] = 0.0;
			Star[15] = x0+0.5*side*(Math.Sin(DegToRad(18))/Math.Cos(DegToRad(36)));
			Star[16] = Star[13];
			Star[17] = 1.0;
			Star[18] = x0;
			Star[19] = Star[13]+side*Math.Sin(DegToRad(36));
			Star[20] = 0.0;
			Star[21] = 2*x0 - Star[15];
			Star[22] = Star[16];
			Star[23] = 1.0;
			Star[24] = 2*x0 - Star[12];
			Star[25] = Star[13];
			Star[26] = 0.0;
			Star[27] = 2*x0 - Star[ 9];
			Star[28] = Star[10];
			Star[29] = 1.0;

            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_3DR;
            dp.MaxCurves = 1;
            dp.MaxPoints = 10;
            dp.NumCurves = 1;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.Title1 = "Data sent to DPlot via dplotlib64.dll";
            dp.Title2 = "Surface Plot with Concave Edges";
            dp.XAxis = "x";
            dp.YAxis = "y";

            cmds = "[Caption(\"DPLOTLIB Concave Surface Plot\")]"+
                "[Contour3D(0)][ContourMethod(0)][ContourGrid(0)][ContourLevels(20,0,1)]"+
                "[DocMaximize()]";
            DocNum = dplot.DPlot_Plot8(ref dp, dummy, Star, cmds);
			if(DocNum > 0)
			{
                int i;
				Border = new double[10*2];
				for(i=0; i<10; i++)
				{
					Border[i*2]   = Star[i*3];
					Border[i*2+1] = Star[i*3+1];
				}
                dplot.DPlot_3DBorder(DocNum, 10, Border);
            }
        }

        private void button6_Click(object sender, EventArgs e)
        {
            int ret;
            StringBuilder ver = new StringBuilder(16);

            ret = dplot.DPlot_GetVersion(ver);
            if (ret == 0)
                MessageBox.Show("DPlot_GetVersion failed.", "Version Number", MessageBoxButtons.OK);
            else
            {
                string text;
                text = "DPlot ";
                if (ret == 2) text += "Jr ";
                text += "Version " + ver.ToString();
                MessageBox.Show(text, "Version Number", MessageBoxButtons.OK);
            }
        }
    }
}