#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "dplot.h"

// DOUBLEPRECISION is used in this example to switch between single- and double-precision versions of 
// the DPLOTLIB functions. Double-precision requires version 1.96 or later of DPlot. In practice there's
// no good reason to do this, the switch is only included for testing purposes and to illustrate the 
// difference, which is apparent in example 7 if you zoom in. To use single-precision, use 
// "#undef DOUBLEPRECISION"

#define DOUBLEPRECISION
#ifdef DOUBLEPRECISION
#define REAL double
#define DPLOT_PLOT dPlot_Plot8
#else
#define REAL float
#define DPLOT_PLOT dPlot_Plot
#endif

int	ExcelDay(int Month,int Day,int Year);

#define DEGTORAD(x) (x/57.295779513082320876798154814105)

// necessary for LCC:
#ifndef RAND_MAX
#define RAND_MAX 0x7fff
#endif

#define PI (REAL)3.1415926535897932384626433832795

typedef struct tagXYZ
{
	REAL x;
	REAL y;
	REAL z;
} XYZ;

typedef struct tagXYZA
{
	REAL x;
	REAL y;
	REAL z;
	REAL a;
} XYZA;

//==============================================================================
char *va(char *format, ...)
{
	va_list	argptr;
	static char	string[1024];
	va_start (argptr, format);
	vsprintf (string, format,argptr);
	va_end (argptr);
	return string;	
}
//==============================================================================
int main()
{
	DPLOT	DPlot;
	int		DocNum;
	int		option;
	int		i, j;
	int		retval;
	DWORD	dwSize;

#ifdef __GNUC__
	HINSTANCE dll;
    if((dll = dPlot_Init()) == NULL)
	{
		printf("LoadLibrary for DPLOTLIB.DLL failed.\n");
		exit(0);
	}
#endif


	dPlot_SetErrorMethod(2);

top:
	printf("Enter 0 to exit\n"
		   "      1 XY plot example\n"
		   "      2 Contour plot of gridded data\n"
		   "      3 Contour plot of randomly-spaced points\n"
		   "      4 Box-and-whisker example\n"
		   "      5 Another XY plot (30 curves)\n"
		   "      6 3D scatter plot\n"
		   "      7 Yet another XY plot (world pop. estimate)\n"
		   "      8 Get DPlot/DPlot Jr version number\n"
		   "      9 Get peak values from document 1\n"
		   "     10 Surface plot with concave border\n"
		   "     11 4D surface plot\n"
		   "     12 4D scatter plot\n"
		   "     13 Another contour plot\n"
		   "     14 2D vector plot\n"
		   "     15 Make movie frames of a 3D/4D plot in 3D view\n"
           "     16 Collatz Conjecture (transparent symbols)\n"
		   "? ");
	scanf("%d",&option);

	switch(option)
	{
	case 1:
		{
			int		NP;
			REAL	*x, *y;

			/* Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
			In this case we've used DATA_XYYY (one X array for one or more Y arrays).
			
			  Since the X values are evenly spaced we could also use
			  DATA_DXY  - X has only 2 elements, DX and X0
			  
				And you can ALWAYS use
				DATA_XYXY - Each curve uses its own X array. This is the only
				option available if the curves have different X values or
				a different number of points.
			*/
			
			NP = 1001;
			x = (REAL *)calloc( NP,   sizeof(REAL));
			y = (REAL *)calloc( NP*2, sizeof(REAL));
			for(i=0; i<NP; i++)
			{
  				x[i]    = (4.f*i)/(NP-1);
				y[i]    = (REAL)sin(PI*x[i]);
				y[i+NP] = (REAL)cos(PI*x[i]);
			}
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_XYYY;
			DPlot.MaxCurves		= 2;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= 2;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			DPlot.LegendX		= 0.05f;
			DPlot.LegendY		= 0.05f;
			DPlot.NP[0]			= NP;
			DPlot.NP[1]			= NP;
			DPlot.LineType[0]	= LINESTYLE_SOLID;
			DPlot.LineType[1]	= LINESTYLE_LONGDASH;
			strcpy(DPlot.Legend[1],"sin({\\sp}x)");	// {\s is DPlot-speak for "use symbol font"
			strcpy(DPlot.Legend[2],"cos({\\sp}x)");
			strcpy(DPlot.Title[0],"Data sent to DPlot via DPLOTLIB.DLL");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			/* The command string can be as complex as you like, limited to 32K characters.
			As an alternative, if your program will be producing many similar plots then
			you might prefer to create a preferences file, edit the file with a text 
			editor to remove unwanted entries, and use the [GetPreferences("filename")]
			command. 
			*/
			DocNum = DPLOT_PLOT(&DPlot,x,y,
				"[Caption(\"DPLOTLIB XY Test\")][DocMaximize()]"
				"[ClearEditFlag()]");
//				"[SetBackgroundImage(\"e:\\dahs\\dplotw\\bitmapto3d.bmp\",1,1,0,0,1,1,0,255,-1)]");

			free(x);
			free(y);
			if(DocNum > 0)
			{
				double	Xinterp, Yinterp;

				printf("\nEnter an X value... ");
				fflush(stdin);
				scanf("%lf",&Xinterp);
				for(i=1; i<=2; i++)
				{
					retval = dPlot_YFromX(DocNum,i,Xinterp,&Yinterp);
					if(retval > 0)
						printf("Curve %d, interpolated Y=%g\n",i,Yinterp);
					else
						printf("Error: return code = %d\n",retval);
				}
			}

		}
		break;
		
	case 2:
		{
			int		Nx, Ny;
			REAL	extents[4];
			REAL	xx, yy;
			REAL	*z;
			char	Metrics[512];
			
			/* Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y) */
			
			Nx = 61;	// Columns
			Ny = 41;	// Rows
			z = (REAL *)calloc( Nx*Ny, sizeof(REAL));
			extents[0] = -3.f;		// xlo
			extents[1] = -2.f;		// ylo
			extents[2] =  3.f;		// xhi
			extents[3] =  2.f;		// yhi
			for(i=0; i<Nx; i++)
			{
				xx = extents[0] + ((extents[2]-extents[0])*i)/(Nx-1);
				for(j=0; j<Ny; j++)
				{
					yy = extents[1] + ((extents[3]-extents[1])*j)/(Ny-1);
					z[i*Ny+j] = (REAL)(sin(xx)*cos(yy));
				}
			}
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3D;
			DPlot.MaxCurves		= Nx-1;		// Size of the grid, in grid cells
			DPlot.MaxPoints		= Ny-1;
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Data sent to DPlot via DPLOTLIB.DLL");
			strcpy(DPlot.Title[1],"z = sin(x)*cos(y)");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			DocNum = DPLOT_PLOT(&DPlot,extents,z,"[Caption(\"DPLOTLIB Contour Test 1\")]"
				"[Contour3D(1)][ContourMethod(0)][ContourGrid(0)][ContourAxes(1)][ContourView(30,20)]"
				"[ContourLevels(20,-1,1)][ContourScales(1,1,1)][ContourLighting(2,0.05)]"
				"[FontPoints(1,10)][FontPoints(6,10)][ZAxisLabel(\"z\")]"
				"[DocMaximize()]");
			free(z);
			dwSize = sizeof(Metrics);
			retval = dPlot_Request(DocNum,"PlotMetrics",Metrics,&dwSize);
			printf("PlotMetrics returns: %s\n", Metrics);
		}
		break;
		
	case 3:
		{
			int		Nz;
			XYZ		*Node;
			
			/* Contour plot with randomly spaced points.
			The random feature of this example is a bit nonsensical since we're using 
			an equation to generate the data. This DPlot feature is much more useful 
			for truly random data and/or data values that aren't organized on a 
			rectangular grid. */
			printf("Enter number of nodes... ");
			fflush(stdin);
			scanf("%d",&Nz);
			Node = (XYZ *)calloc( Nz, sizeof(XYZ));
			for( i=0; i < Nz; i++ ) {
				Node[i].x = -PI + 2*PI * (float)rand()/(float)RAND_MAX;
				Node[i].y = -PI + 2*PI * (float)rand()/(float)RAND_MAX;
				Node[i].z = (float)(sin(Node[i].x) * cos(Node[i].y));
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3DR;
			DPlot.MaxCurves		= 1;
			DPlot.MaxPoints		= Nz;
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Data sent to DPlot via DPLOTLIB.DLL");
			strcpy(DPlot.Title[1],"z = sin(x)*cos(y)");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			DocNum = DPLOT_PLOT(&DPlot,NULL,(REAL *)Node,"[Caption(\"DPLOTLIB Contour Test 2\")]"
				"[Contour3D(0)][ContourGrid(0)][ContourMethod(1)][ContourLevels(21,-1,1)]"
				"[FontPoints(1,10)][FontPoints(6,10)][DocMaximize()]");
			free(Node);
		}
		break;

	case 4:
		{
			int		Ny;
			REAL	*y, *py;
			
			/* Box-and-whisker plot - 3 groups */
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_1D;
			DPlot.MaxCurves		= 3;
			DPlot.MaxPoints		= 65;
			DPlot.NumCurves		= 3;
			DPlot.LegendX		= 0.02f;
			DPlot.LegendY		= 0.7f;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Data sent to DPlot via DPLOTLIB.DLL");
			strcpy(DPlot.YAxis,"Amplitude");
			
			// Number of points in each group is 50 +/- 15:
			Ny = 0;
			for(i=0; i < (int)DPlot.NumCurves; i++)
			{
				DPlot.NP[i] = 50 + (int)(30*(-0.5f + (REAL)rand()/(REAL)RAND_MAX));
				Ny += DPlot.NP[i];
				sprintf(DPlot.Legend[i+1],"Group %c",65+i );
			}
			
			// Generate group data, centered around 5.0 with progressively larger variations
			y = (REAL *)calloc( Ny, sizeof(REAL));
			py = y;
			for( i=0; i < (int)DPlot.NumCurves; i++)
			{
				for(j=0; j < (int)DPlot.NP[i]; j++)
				{
					*py = 5.f + (REAL)(i+1) * (-0.5f + (REAL)rand()/(REAL)RAND_MAX);
					py++;
				}
			}
			
			DocNum = DPLOT_PLOT(&DPlot,NULL,y,"[Caption(\"DPLOTLIB Box-and-whisker Test\")]"
				"[Stat_PlotType(0)][Stat_GroupInfo(15,4)][Stat_BWOptions(1,1,0,1,1.5,2,3)]"
				"[Stat_GrandMean(3,30)][FontPoints(1,10)][FontPoints(6,10)][DocMaximize()]");
			free(y);

		}
		break;

	case 5:
		{
			int		NC, NP;
			REAL	*x, *y;
			
			/* Simulate a square wave with a slew of sine waves. First 29 curves are
			sine waves, last curve is the addition of 1st 29.
			
			  Depending on which operating system you're using, attempting this with
			DPLOT_DDE_VERSION=2 will either crash or produce very odd results.    */
			
			NC = 30;
			NP = 1001;
			x = (REAL *)calloc( NP,   sizeof(REAL));
			y = (REAL *)calloc( NP*NC, sizeof(REAL));
			for(i=0; i<NP; i++)
			{
				x[i] = (2*PI*i)/(NP-1);
				y[i+(NC-1)*NP] = 0.f;
				for(j=0; j<NC-1; j++)
				{
					y[i+j*NP] = (REAL)sin((j*2+1)*x[i])/(j*2+1);
					y[i+(NC-1)*NP] += y[i+j*NP];
				}
				y[i+(NC-1)*NP] *= 4.0/PI;
			}
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_XYYY;
			DPlot.MaxCurves		= NC;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= NC;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			DPlot.LegendX		= 0.05f;
			DPlot.LegendY		= 0.7f;
			for(j=0; j<NC; j++)
			{
				DPlot.NP[j]	= NP;
				DPlot.LineType[j] = LINESTYLE_SOLID;
  				if(j < NC-1)
  					sprintf(DPlot.Legend[j+1],"sin(%dx)/%d",j*2+1,j*2+1);
			}
  			strcpy(DPlot.Legend[NC],"4/{\\sp}(sin(x)+sin(3x)/3+sin(5x)/5+...)");
			strcpy(DPlot.Title[0],"Square wave approximation");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			/* The command string can be as complex as you like, limited to 32K characters.
			As an alternative, if your program will be producing many similar plots then
			you might prefer to create a preferences file, edit the file with a text 
			editor to remove unwanted entries, and use the [GetPreferences("filename")]
			command. 
			*/
			retval = DPLOT_PLOT(&DPlot,x,y,"[ManualScale(0,-1.25,6.5,1.25)][TickInterval(1,0.5,0.25)]"
				"[LineWidth(-2,0)][Caption(\"DPLOTLIB XY Test\")][DocMaximize()]"
				"[ClearEditFlag()]");
			free(x);
			free(y);
			printf("return value from DPlot_Plot=%d\n",retval);
		}
		break;
		
	case 6:
		{
			/* 3D scatter plot - random points on 3 spheres. */

			int		NC, Nz;
			REAL	zz, radius, theta, phi;
			REAL	theta_inc;
			REAL	xmin, xmax, ymin, ymax, zmin, zmax;
			REAL	xx, yy;
			XYZ		*Node;
			char	cmd[512];

			memset(&DPlot,0,sizeof(DPlot));
			Nz = 1000;      // Number of points in each of the NC data sets. Data sets CAN have
							// differing numbers of points. In either case, the Node array is 
							// packed with no unused nodes. For example the first point in the
							// 2nd data set should immediately follow the last point in the 1st
							// data set.
			NC = 3;
			srand( (unsigned)time( NULL ) );
			Node = (XYZ *)calloc( NC*Nz, sizeof(XYZ));
			xmin = xmax = 0.;
			ymin = ymax = 0.;
			zmin = zmax = 0.;
			theta_inc = (REAL)(2.0*sqrt((REAL)Nz));
			for(j=0; j<NC; j++)
			{
				xx = (REAL)(-5+10.*rand()/RAND_MAX);
				yy = (REAL)(-5+10.*rand()/RAND_MAX);
				zz = (REAL)(-5+10.*rand()/RAND_MAX);
				radius = (REAL)(4.+4.*rand()/RAND_MAX);
/*				if(j == 0)
				{
					xx = -9.;
					yy = 0.;
					zz = 5.;
					radius = 4;
				} else if (j == 2)
				{
					xx = 9.;
					radius = 4;
				} else
				{
					xx = 0;
					radius = 5;
				} */
				xmin = min(xmin,xx-radius);
				xmax = max(xmax,xx+radius);
				ymin = min(ymin,yy-radius);
				ymax = max(ymax,yy+radius);
				zmin = min(zmin,zz-radius);
				zmax = max(zmax,zz+radius);
				for( i=j*Nz; i < (j+1)*Nz; i++ ) {
					phi = -PI/2+PI*(REAL)(i-j*Nz)/(REAL)Nz;
					theta = theta_inc*PI*(REAL)(i-j*Nz)/(REAL)Nz;
					Node[i].x = xx + (REAL)(radius*cos(theta)*cos(phi));
					Node[i].y = yy + (REAL)(radius*sin(theta)*cos(phi));
					Node[i].z = zz + (REAL)(radius*sin(phi));
				}
				
				sprintf(DPlot.Legend[j+1],"Sphere %d",j+1);
				DPlot.NP[j] = Nz;
			}
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3DS;
			DPlot.MaxCurves		= NC;
			DPlot.MaxPoints		= Nz;
			DPlot.NumCurves		= NC;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Legend[0],"Legend");
			strcpy(DPlot.Title[0],"DPLOTLIB - Points on Spheres");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			xmin = (REAL)floor(xmin);
			xmax = (REAL)ceil(xmax);
			ymin = (REAL)floor(ymin);
			ymax = (REAL)ceil(ymax);
			zmin = (REAL)floor(zmin);
			zmax = (REAL)ceil(zmax);

			sprintf(cmd,"[Caption(\"DPLOTLIB Test 6\")]"
				"[TickInterval(1,1,1,1)][ManualScale(%g,%g,%g,%g,%g,%g)][ColorBasis(1)][ContourColorScheme(2)]"
				"[ContourCustomColors(255,65535,16711680)][ContourScales(1,1,1)][SymbolOutline(1)]"
				"[FontPoints(1,10)][FontPoints(6,10)][DocMaximize()]",
				xmin,ymin,xmax,ymax,zmin,zmax);
			DocNum = DPLOT_PLOT(&DPlot,NULL,(REAL *)Node,cmd);
			free(Node);
		}
		break;

	case 7:
		{
#define NUMDAYS 10
#define SAMPLESPERDAY (24*60*6)		// 10 second intervals
#define REPLACEMENTRATE 0.013

			int		NP;
			double	JanOne04Pop=6393657063.;
			REAL	BaseDay;
			REAL	CurrentDate;
			time_t	ltime;
			struct	tm *today;
			REAL	elapsed_time;
			REAL	*x, *y;

			BaseDay = (REAL)ExcelDay(1,1,2004);
			time( &ltime );
			today = localtime( &ltime );
			CurrentDate = (REAL)ExcelDay(today->tm_mon+1,today->tm_mday,today->tm_year+1900) +
				(REAL)today->tm_hour/24 + (REAL)today->tm_min/(24*60);

			// We'll estimate the world population every 30 seconds for NUMDAYS days
			NP = NUMDAYS*SAMPLESPERDAY + 1;
			x = (REAL *)calloc( NP, sizeof(REAL));
			y = (REAL *)calloc( NP, sizeof(REAL));
			for(i=0; i<NP; i++)
			{
				x[i] = CurrentDate + (REAL)i/(REAL)SAMPLESPERDAY;
				elapsed_time = x[i] - BaseDay;
				y[i] = (REAL)floor( JanOne04Pop * exp(REPLACEMENTRATE*elapsed_time/365.25) );
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_XYXY;
			DPlot.MaxCurves		= 1;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			DPlot.NP[0]			= NP;
			DPlot.LineType[0]	= LINESTYLE_SOLID;
			strcpy(DPlot.Title[0],"World Population Estimate");
			sprintf(DPlot.Title[1],"based on Jan 1, 2004 population of %.10g",JanOne04Pop);
			sprintf(DPlot.Title[2],"and a replacement rate of %g%%",100*REPLACEMENTRATE);
			strcpy(DPlot.XAxis,"Date");
			strcpy(DPlot.YAxis,"Population");

			retval = DPLOT_PLOT(&DPlot,x,y,"[Caption(\"DPLOTLIB XY Test\")]"
				"[DateFormat(\"M/dd/yy\")][TimeFormat(\"h:mm:ss\")]"
				"[NumberFormat(0,6)][NumberFormat(1,11)][ClearEditFlag()]");
			free(x);
			free(y);
			printf("return value from DPlot_Plot=%d\n",retval);
		}
		break;

	case 8:
		{
			int		result;
			char	version[32];

			result = dPlot_GetVersion(version);
			if(!result)
				printf("DPlot_GetVersion failed!\n");
			else
				printf("%s Version %s\n",(result==1 ? "DPlot" : "DPlot Jr"),version);
		}
		break;

	case 9:
		{
			char	szPeaks[1024];
			DWORD	dwSize;
			
			dwSize = sizeof(szPeaks);
			retval = dPlot_Request(1,"ListPeaks",szPeaks,&dwSize);
			if(retval > 0)
				printf("%s\n",szPeaks);
			else if(retval == -3)
				printf("insufficient buffer size, %d required.\n",dwSize);
			else
				printf("return value of DPlot_Request=%d\n",retval);
		}
		break;

	case 10:
		{
			// surface plot with concave edges
			XYZ		Star[10];	
			REAL	x0, y0, side;
			x0 = 0;
			y0 = 0;
			side = 10.0;

			Star[0].x = x0 - (REAL)(0.5*side);
			Star[0].y = y0 - (REAL)(0.5*side*tan(DEGTORAD(54)));
			Star[0].z = 0.0;
			Star[1].x = x0;
			Star[1].y = Star[0].y+(REAL)(0.5*side*tan(DEGTORAD(36)));
			Star[1].z = 1.0;
			Star[2].x = x0 + (REAL)(0.5*side);
			Star[2].y = Star[0].y;
			Star[2].z = 0.0;
			Star[3].x = Star[2].x-(REAL)(0.5*side*(sin(DEGTORAD(18))/cos(DEGTORAD(36))));
			Star[3].y = Star[2].y+(REAL)(0.5*side*(cos(DEGTORAD(18))/cos(DEGTORAD(36))));
			Star[3].z = 1.0;
			Star[4].x = Star[2].x+side*(REAL)sin(DEGTORAD(18));
			Star[4].y = Star[2].y+side*(REAL)cos(DEGTORAD(18));
			Star[4].z = 0.0;
			Star[5].x = x0+(REAL)(0.5*side*(sin(DEGTORAD(18))/cos(DEGTORAD(36))));
			Star[5].y = Star[4].y;
			Star[5].z = 1.0;
			Star[6].x = x0;
			Star[6].y = Star[4].y+side*(REAL)sin(DEGTORAD(36));
			Star[6].z = 0.0;
			Star[7].x = 2*x0 - Star[5].x;
			Star[7].y = Star[5].y;
			Star[7].z = 1.0;
			Star[8].x = 2*x0 - Star[4].x;
			Star[8].y = Star[4].y;
			Star[8].z = 0.0;
			Star[9].x = 2*x0 - Star[3].x;
			Star[9].y = Star[3].y;
			Star[9].z = 1.0;
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3DR;
			DPlot.MaxCurves		= 1;
			DPlot.MaxPoints		= 10;
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Surface Plot with Concave Edges");
			strcpy(DPlot.Title[1],"See {\\iDefine Boundary} command on {\\iOptions} menu");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
				
			DocNum = DPLOT_PLOT(&DPlot,NULL,(REAL *)Star,"[Caption(\"DPLOTLIB Concave Surface Plot\")]"
					"[Contour3D(0)][ContourMethod(0)][ContourGrid(0)][ContourLevels(20,0,1)]"
					"[DocMaximize()]");
			if(DocNum)
			{
				double border[10][2];
				for(i=0; i<10; i++)
				{
					border[i][0] = Star[i].x;
					border[i][1] = Star[i].y;
				}
				dPlot_3DBorder(DocNum,10,&border[0][0]);
			}
		}
		break;
	
	case 11:
		{
			XYZA	*Node;
			int		Nx, Ny, k;
			REAL	x, y, z;

			// 4D Contour plot of Z values on a rectangular grid.
			
			Nx = Ny = 121;	// Rows and columns of data
			Node = (XYZA *)calloc( Nx*Ny, sizeof(XYZA));
			for(i=0, k=0; i<Nx; i++)
			{
				x = 5 + 30*(REAL)i/(REAL)(Nx-1);
				for(j=0; j<Ny; j++, k++)
				{
					Node[k].x = x;
					Node[k].y = y = 5 + 30*(REAL)j/(REAL)(Ny-1);
					if(x==20. && y==20.)
						z = sin(1.0);
					else
						z = sin(sqrt((x-20.)*(x-20.)+(y-20.)*(y-20.)))/sqrt((x-20.)*(x-20.)+(y-20.)*(y-20.));
					z += 1.5;
					z *= 100.;
					Node[k].z = z;
					Node[k].a = 10. * (1.1 + sin((x-20.)/5.)*cos((y-20.)/10.));
				}
			}
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_4D;
			DPlot.MaxCurves		= 1;
			DPlot.MaxPoints		= Nx*Ny;
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Data sent to DPlot via DPLOTLIB.DLL");
			strcpy(DPlot.Title[1],"z = 100 * (sin(sqrt((x-20.){\\u2}+(y-20.){\\u2}))/sqrt((x-20.){\\u2}+(y-20.){\\u2})+1.5)");
			strcpy(DPlot.Title[2],"a = 10. * (1.1 + sin((x-20.)/5.)*cos((y-20.)/10.))");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			DocNum = DPLOT_PLOT(&DPlot,NULL,(REAL *)Node,"[Caption(\"DPLOTLIB 4D Surface Plot Test\")]"
				"[Contour3D(1)][ContourMethod(0)][ContourGrid(0)][ContourAxes(1)][ContourView(118,28)]"
				"[ContourLevels(22,0,22)][ContourScales(1,1,0.2)][ContourLighting(0,0.05)]"
				"[ContourColorScheme(2)][ContourCustomColors(0,0x00FF047D,0x000B1EB3,0x000068DC,0x0000FAFE)]"
				"[FontPoints(1,10)][FontPoints(4,14)][FontPoints(5,14)][FontPoints(6,10)][FontPoints(8,14)]"
				"[ZAxisLabel(\"z\")][AAxisLabel(\"Amplitude\")][DocMaximize()]");
			free(Node);
		}
		break;

	case 12:
		{
			/* 4D scatter plot - random points on 3 spheres. */

			int		NC, Nz;
			REAL	zz, radius, theta, phi;
			REAL	theta_inc;
			REAL	xmin, xmax, ymin, ymax, zmin, zmax;
			REAL	xx, yy;
			XYZA	*Node;
			char	cmd[512];

			memset(&DPlot,0,sizeof(DPlot));
			Nz = 1000;      // Number of points in each of the NC data sets. Data sets CAN have
							// differing numbers of points. In either case, the Node array is 
							// packed with no unused nodes. For example the first point in the
							// 2nd data set should immediately follow the last point in the 1st
							// data set.
			NC = 3;
			srand( (unsigned)time( NULL ) );
			Node = (XYZA *)calloc( NC*Nz, sizeof(XYZA));
			xmin = xmax = 0.;
			ymin = ymax = 0.;
			zmin = zmax = 0.;
			theta_inc = (REAL)(2.0*sqrt((double)Nz));
			for(j=0; j<NC; j++)
			{
				xx = (REAL)(-10.+20.*rand()/RAND_MAX);
				yy = (REAL)(-10.+20.*rand()/RAND_MAX);
				zz = (REAL)(-10.+20.*rand()/RAND_MAX);
				radius = (REAL)(4.+4.*rand()/RAND_MAX);
				xmin = min(xmin,xx-radius);
				xmax = max(xmax,xx+radius);
				ymin = min(ymin,yy-radius);
				ymax = max(ymax,yy+radius);
				zmin = min(zmin,zz-radius);
				zmax = max(zmax,zz+radius);
				for( i=j*Nz; i < (j+1)*Nz; i++ ) {
					phi = -PI/2+PI*(REAL)(i-j*Nz)/(REAL)Nz;
					theta = theta_inc*PI*(REAL)(i-j*Nz)/(REAL)Nz;
					Node[i].x = xx + (REAL)(radius*cos(theta)*cos(phi));
					Node[i].y = yy + (REAL)(radius*sin(theta)*cos(phi));
					Node[i].z = zz + (REAL)(radius*sin(phi));
					Node[i].a = Node[i].x;
				}
				sprintf(DPlot.Legend[j+1],"Sphere %d",j+1);
				DPlot.NP[j] = Nz;
			}
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_4DS;
			DPlot.MaxCurves		= NC;
			DPlot.MaxPoints		= Nz;
			DPlot.NumCurves		= NC;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Legend[0],"Legend");
			strcpy(DPlot.Title[0],"DPLOTLIB - Points on Spheres");
			strcpy(DPlot.Title[1],"Amplitude=X");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			xmin = (REAL)floor(xmin);
			xmax = (REAL)ceil(xmax);
			ymin = (REAL)floor(ymin);
			ymax = (REAL)ceil(ymax);
			zmin = (REAL)floor(zmin);
			zmax = (REAL)ceil(zmax);

			sprintf(cmd,"[Caption(\"DPLOTLIB Test 12\")]"
				"[TickInterval(1,1,1,1)][ManualScale(%g,%g,%g,%g,%g,%g)][ColorBasis(0)][ContourColorScheme(2)]"
				"[ContourCustomColors(255,65535,16711680)][ContourScales(1,1,1)]"
				"[FontPoints(1,10)][FontPoints(6,10)][DocMaximize()][SymbolType(-1,288)]"
				"[SymbolSize(1,100)][SymbolSize(2,150)][SymbolSize(3,200)][SymbolOutline(1)]",
				xmin,ymin,xmax,ymax,zmin,zmax);
			DocNum = DPLOT_PLOT(&DPlot,NULL,(REAL *)Node,cmd);
			free(Node);
		}
		
		break;
	case 13:
		{
			int		Nx, Ny;
			REAL	extents[4];
			REAL	xx, yy;
			REAL	*z;
			REAL	Vc, Vs, Vnet, Vf;
			REAL	theta;
			
			Nx = 361;	// Columns
			Ny = 161;	// Rows
			z = (REAL *)calloc( Nx*Ny, sizeof(REAL));
			extents[0] = 0.0;		// xlo
			extents[1] = 0.0;		// ylo
			extents[2] = 360.0;		// xhi
			extents[3] = 1.0;		// yhi
			for(i=0; i<Nx; i++)
			{
				xx = extents[0] + ((extents[2]-extents[0])*i)/(Nx-1);
				for(j=0; j<Ny; j++)
				{
					yy = extents[1] + ((extents[3]-extents[1])*j)/(Ny-1);
					Vc = (REAL)(1.0-yy*cos(DEGTORAD(xx)));
					Vs = (REAL)(yy*sin(DEGTORAD(xx)));
					Vnet = (REAL)sqrt(Vc*Vc + Vs*Vs);
					// Borland C doesn't handle atan2(0,0). Others do.
					if(Vnet > 0)
						theta = (REAL)atan2(Vs*Vs,Vc*Vc);
					else
						theta = 0.;
					Vf = Vnet*Vnet * (REAL)fabs(cos(theta));
					if(Vc < 0)
						z[i*Ny+j] = (REAL)(-Vf);
					else
						z[i*Ny+j] = (REAL)Vf;
				}
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3D;
			DPlot.MaxCurves		= Nx-1;		// Size of the grid, in grid cells
			DPlot.MaxPoints		= Ny-1;
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Relative Drag Force for a Spherical Object");
			strcpy(DPlot.Title[1],"Drag{\\dactual}/Drag{\\dno wind}\n");
			strcpy(DPlot.XAxis,"Wind Direction, degrees");
			strcpy(DPlot.YAxis,"Wind Speed/Ground Speed");
			
			DocNum = DPLOT_PLOT(&DPlot,extents,z,"[Caption(\"DPLOTLIB Contour Test 13\")]"
				"[Contour3D(0)][ContourMethod(0)][ContourGrid(0)][ContourAxes(1)]"
				"[ContourLevels(40,0,4)][ContourScales(1,360,1)][ContourLighting(2,0.05)]"
				"[ManualScale(0,0,360,1,0,4)][TickInterval(1,10,0.1,1)]"
				"[FontPoints(1,10)][FontPoints(6,10)][ZAxisLabel(\"z\")]"
				"[XYLabel(0,1,0,\"Tailwind\")][XYLabel(180,1,0,\"Headwind\")][XYLabel(360,1,0,\"Tailwind\")]"
				"[DocMaximize()][ClearEditFlag()]");
			free(z);
		}
		break;
	case 14:
		{
			int		Nx, Ny, NP;
			int		k;
			REAL	xlo, ylo, xhi, yhi;
			REAL	xx, yy;
			REAL	*x, *y;
			REAL	alpha;
			
  			Nx = 37;	// Columns
  			Ny = 16;	// Rows
			NP = Nx*Ny;
			x = (REAL *)calloc( NP, sizeof(REAL));
			y = (REAL *)calloc(3*NP, sizeof(REAL));

			xlo = 0.0;		// xlo
			ylo = 0.0;		// ylo
			xhi = 360.0;	// xhi
			yhi = 1.5;		// yhi
			k = 0;
			for(i=0; i<Nx; i++)
			{
				xx = xlo + ((xhi-xlo)*i)/(Nx-1);
				for(j=0; j<Ny; j++)
				{
					yy = ylo + ((yhi-ylo)*j)/(Ny-1);
					x[k] = xx;
					y[k] = yy;
					alpha = xx;
					if(alpha < 0) alpha += 360.;
					y[k+NP] = (REAL)(PI-atan2(yy*sin(DEGTORAD(alpha)),1.0-yy*cos(DEGTORAD(alpha))));
					if(y[k+NP] < 0.0) y[k+NP] += (REAL)(2.*PI);
					if(y[k+NP] >= 2.*PI) y[k+NP] -= (REAL)(2.*PI);
					y[k+2*NP] = (REAL)sqrt(1.0-2*yy*cos(DEGTORAD(alpha))+yy*yy);
					k++;
				}
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_XYYY;
			DPlot.MaxCurves		= 3;
			DPlot.MaxPoints		= NP;
			DPlot.NumCurves		= 3;
			DPlot.NP[0] = DPlot.NP[1] = DPlot.NP[2] = NP;
			DPlot.SymbolType[0] = SYMBOLSTYLE_DOT;
			DPlot.SymbolType[1] = SYMBOLSTYLE_DOT;
			DPlot.SymbolType[2] = SYMBOLSTYLE_DOT;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Air Speed/Ground Speed\\n");
			strcpy(DPlot.XAxis,"Wind Direction, degrees");
			strcpy(DPlot.YAxis,"Wind Speed/Ground Speed");
			
			DocNum = DPLOT_PLOT(&DPlot,x,y,"[Caption(\"DPLOTLIB Contour Test 14\")]"
				"[ManualScale(-10,-0.1,370,1.6)]"
				"[TickInterval(1,10,0.1)]"
				"[FontPoints(1,10)][FontPoints(6,10)][AxisType(2)]"
				"[XYLabel(0,1,0,\"Tailwind\")][XYLabel(90,1,0,\"Left\")][XYLabel(180,1,0,\"Headwind\")]"
				"[XYLabel(270,1,0,\"Right\")][XYLabel(360,1,0,\"Tailwind\")]"
				"[VectorPlot(1,2,3,0x0000FF00,0x000000FF,100,500)][VectorPlotOptions(6,150,20)]"
				"[DocMaximize()][ClearEditFlag()]");
			free(x);
			free(y);
		}
		break;
	case 15:		// Make movie frames of current 3D plot
		{
			/* NOTE: There's currently (2.2.4.1) no way to check for whether a 3D/4D plot is
			   displayed in a 3D view. This routine will work with 2D views w/o a problem, but
			   all frames will be identical. */
			char	szCmd[128];
			char	szValue[32];
			DWORD	dwSize;
			int		az_increment;
			int		width, height;
			char	szDestination[256];
			int		DocType;

			dwSize = sizeof(szValue);
			if(dPlot_Request(0,"ActiveDocument",szValue,&dwSize) < 0)
			{
				printf("Error retrieving active document index from DPlot.\n");
				break;
			}
			DocNum = atol(szValue);
			if(DocNum <= 0)
			{
				printf("No document is open. To use this feature you need an open 3D or 4D plot with a 3D view in DPlot\n");
				break;
			}
			dwSize = sizeof(szValue);
			if(dPlot_Request(DocNum,"DocumentType",szValue,&dwSize) < 0)
			{
				printf("Error retrieving document type (random 3D points, etc.) from document index %d.\n",DocNum);
				break;
			}
			DocType = atol(szValue);
			if((DocType != DATA_3D ) &&  // Z values on a rectangular grid
			   (DocType != DATA_3DR) &&  // Random X,Y,Z values
			   (DocType != DATA_3DS) &&  // 3D scatter plot
			   (DocType != DATA_4D ) &&  // 4D surface plot
			   (DocType != DATA_4DS)     // 4D scatter plot
			  )
			{
				printf("This feature only works with 3D/4D documents.\nThe currently active document is type %d.\n",DocType);
				break;
			}
			printf("\nEnter azimuth increment in degrees (0 to abort)... ");
			fflush(stdin);
			scanf("%d",&az_increment);
			if(az_increment == 0) break;
			printf("\nEnter desired bitmap width and height, in pixels (640x480 suggested)... ");
			scanf("%d,%d",&width,&height);
			printf("\nEnter destination path for bitmap images... ");
			scanf("%s",szDestination);
			i = strlen(szDestination);
			if(szDestination[i-1] == '\\') szDestination[i-1] = '\0';
			printf("\n");

			sprintf(szCmd,"[SetClientRect(1,0,0,%d,%d)][SetPluginImageDims(%d,%d,24,0,0)][SetImageCrop(0)][Preserve3DScaleFactor()]",
				width-1,height-1,width,height);
			dPlot_Command(DocNum,szCmd);
			for(i=0; i<360; i+=az_increment)
			{
				sprintf(szCmd,"[ContourView(%d)]",i);
				dPlot_Command(DocNum,szCmd);
				sprintf(szCmd,"[FileSaveAs(\"BMP Picture\",\"%s\\%03d.bmp\")]",szDestination,i);
				dPlot_Command(DocNum,szCmd);
				printf("%d\n",i);
			}
			dPlot_Command(DocNum,"[Preserve3DScaleFactor(0)]");
		}
		break;

	case 16:
		{
			// Collatz Conjecture
			int		NP;
			LONGLONG k;			// LONGLONG is required for NP >= 113383. Otherwise int works fine.
			REAL	*x, *y;

			NP = 100000;
			x = (REAL *)calloc( NP, sizeof(REAL));
			y = (REAL *)calloc( NP, sizeof(REAL));
			for(i=1; i<=NP; i++)
			{
				j = 0;
				k = i;
				while(k != 1)
				{
					if(k % 2 == 0)
						k /= 2;
					else
						k = k*3 + 1;
					j++;
				}
				x[i-1] = (REAL)i;
				y[i-1] = (REAL)j;
			}
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_XYXY;
			DPlot.MaxCurves		= 1;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			DPlot.LegendX		= 0.05f;
			DPlot.LegendY		= 0.05f;
			DPlot.NP[0]			= NP;
			DPlot.LineType[0]	= LINESTYLE_NONE;
			DPlot.SymbolType[0] = 288;
			strcpy(DPlot.Title[0],"Collatz Conjecture");
			strcpy(DPlot.YAxis,"Number of iterations");

			DocNum = DPLOT_PLOT(&DPlot,x,y,
				va("[Caption(\"DPLOTLIB Test\")][DocMaximize()]"
				"[ClearEditFlag()][SymbolSizeUnits(0)][SymbolSize(1,100)][SymbolOpacity(1,10)]"
				"[TextNoteEx(\"{\\bCollatz Conjecture:}\\n"
							"Take any natural number {\\in}. If {\\in} is even, divide it by 2\\n"
							"to get {\\in}/2, if {\\in} is odd multiply it by 3 and add 1 to\\n"
							"obtain 3{\\in} + 1. Repeat the process indefinitely. The\\n"
							"conjecture is that no matter what number you start with,\\n"
							"you will always eventually reach 1. This example plots\\n"
							"the number of iterations required to reach 1 for all\\n"
							"numbers between 1 and %d (and also shows off \\n"
							"DPlot's partially transparent symbols feature).\",0,0,0,0)]",NP)
				);
			free(x);
			free(y);
		}
		break;
/*
	Whoops. If you're looking at this you've found my secret stash of
	additional examples that aren't especially interesting; I was just
	testing things and don't want to throw these out.
*/
	case 17:
		{
			// Forum questions (and answers!)
			int	NP;
			REAL	*x, *y;
			NP = 2;
			x = (REAL *)calloc(2,sizeof(REAL));
			y = (REAL *)calloc(2*3,sizeof(REAL));
			x[0]=1;
			x[1]=0;
			y[0]=55;
			y[1]=23;
			y[2]=65;
			y[3]=40;
			y[4]=80;
			y[5]=10;
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_DXY;
			DPlot.MaxCurves		= 3;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= 3;
			DPlot.Scale			= SCALE_BARCHART;
			DPlot.NP[0]			= NP;
			DPlot.NP[1]         = NP;
			DPlot.NP[2]         = NP;
			strcpy(DPlot.Legend[1],"2007");
			strcpy(DPlot.Legend[2],"2008");
			strcpy(DPlot.Legend[3],"2009");
			strcpy(DPlot.Title[0],"Bar Chart Question from Forums");
			DocNum = DPLOT_PLOT(&DPlot,x,y,
				"[Caption(\"Bar Chart Question from Forums\")]"
				"[BarChartStyle(1)][BarWidth(1,0.75)]"
				"[XYLabel(0.5,0,0,\"Apples\")]"
				"[XYLabel(1.5,0,0,\"Pears\")]"
				"[NumberFormat(0,12)]"
				"[ManualScale(0,0,2,90)]"	// This isn't strictly necessary, but w/o it the y=80 bar goes to the top of the plot.
				"[Size(1,6,6,1)]"
				// Standard Excel colors, just because:
				"[BkPlotColor(0xC0C0C0)][AxisColor(128,128,128)][Color(0,128,128,128)][Color(1,154,153,255)][Color(2,153,51,101)][Color(3,255,255,203)]"
				"[LegendParams(0.2,,1,1,0.75,0.08,,1,0xFFFFFF)]"
				"[LineWidth(-1,30)]"
				);
			free(x);
			free(y);

		}
		break;
	case 18:
		{
			int		Nz;
			XYZ		*Node;
			
			printf("Enter number of nodes... ");
			fflush(stdin);
			scanf("%d",&Nz);
			Node = (XYZ *)calloc( Nz, sizeof(XYZ));
			for( i=0; i < Nz; i++ ) {
				Node[i].x = -PI + 2*PI * (float)rand()/(float)RAND_MAX;
				Node[i].y = -PI + 2*PI * (float)rand()/(float)RAND_MAX;
				Node[i].z = (float)(sin(Node[i].x) * cos(Node[i].y));
			}
			memset(&DPlot,0,sizeof(DPlot));
			DocNum = dPlot_Command(0,"[FileNew(3)]");
			dPlot_AddData8(DocNum, DATA_3DR, Nz, 0, (double *)&Node[0], (double *)&Node[0]);
			free(Node);

		}
		break;

	case 19:
		{
			int		lon, lat, Nz, step;
			XYZ		*Node;

			printf("\nEnter lat, lon step in degrees...");
			fflush(stdin);
			scanf("%d",&step);

			Nz = (360*90)/(step*step) + 1;

			i = 0;
			Node = (XYZ *)calloc( Nz, sizeof(XYZ));
			for(lat=0; lat<90; lat += step)
			{
				for(lon=0; lon<360; lon += step)
				{
					Node[i].x = cos(lon*PI/180.)*cos(lat*PI/180.);
					Node[i].y = sin(lon*PI/180.)*cos(lat*PI/180.);
					Node[i].z = sin(lat*PI/180.);
					i++;
				}
			}
			Node[i].x = Node[i].y = 0.;
			Node[i].z = 1.0;


			// Unit hemisphere
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version = DPLOT_DDE_VERSION;
			DPlot.DataFormat = DATA_3DR;
			DPlot.MaxCurves = 1;
			DPlot.MaxPoints = Nz;
			DPlot.NumCurves = 1;
			DPlot.Scale = SCALE_LINEARX_LINEARY;
			DPlot.NP[0] = Nz;
			DPLOT_PLOT(&DPlot,NULL,(REAL *)Node,"[Caption(\"DPLOTLIB Unit Hemisphere\")]");
			free(Node);
		}
		break;

	case 20:
		{
		// Get a list of all available DPlot DDE topics and display them on the screen.
			char	szTopics[2048];
			DWORD	dwSize;

			dwSize = sizeof(szTopics);
			retval = dPlot_Request(0,"Topics",szTopics,&dwSize);
			printf("%s\n",szTopics);
			break;
		}
		break;
	case 21:
		{
			// Mobius strip, 3D scatter plot
			int		Nz, angle_degrees;
			double	u, v;
			char	cmd[512];
			XYZ		*Node;

			Nz = 21*360;
			Node = (XYZ *)calloc( Nz, sizeof(XYZ));
			i = 0;
			for(u=-1.0; u<=1.0; u+=0.1)
			{
				for(angle_degrees=0; angle_degrees<360; angle_degrees++)
				{
					v = (double)angle_degrees*PI/180;
					Node[i].x = (3+u*cos(v/2))*cos(v);
					Node[i].y = (3+u*cos(v/2))*sin(v);
					Node[i].z = u*sin(v/2);
					i++;
				}
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3DS;
			DPlot.MaxCurves		= 1;
			DPlot.MaxPoints		= Nz;
			DPlot.NumCurves		= 1;
			DPlot.NP[0]         = Nz;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"DPLOTLIB - Points on Mobius Strip");
			sprintf(cmd,"[Caption(\"DPLOTLIB Test 20\")][DocMaximize()]");
			DocNum = DPLOT_PLOT(&DPlot,NULL,(REAL *)Node,cmd);
			free(Node);
		}
		break;

	case 22:
		dPlot_Command(1,"[SelectCurve(1)][EditSmooth(0,10)]");
		break;
	case 0:
#ifdef __GNUC__
	    FreeLibrary(dll);
#endif
		return 0;
		
	}
	goto top;	
}

//======================================================================================================
int	ExcelDay(int Month,int Day,int Year)
{
	if(Year < 1900 || Year > 2078)
		return -1;
	else
	{
		if( Month > 2 )
			return (int)(365.25*Year)+(int)(30.6001*(Month+1))+Day-694037;
		else
			return (int)(365.25*(Year-1))+(int)(30.6001*(Month+13))+Day-694037;
	}
}
