using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Timers;
using System.Windows.Forms;
using System.Runtime.InteropServices;


namespace test5
{
    public partial class Form1 : Form
    {
        [System.Runtime.InteropServices.DllImport("kernel32.dll")]
        public static extern int GetTickCount();

        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();

        double TimeStep;
        IntPtr hwndDPlot;
        int DocNum;
        int StartTicks;
        int StopTicks;
        int Running;
        int NumSkippedFrames = 0;
        int busy = 0;
        int xLast, yLast;
        int ForceRedraw;
        int dxPic, dyPic;   // used for resizing
        Rectangle rcPic;
        dplot.DPLOT_PLOTMETRICSEX DPM;

        public Form1()
        {
            InitializeComponent();
        }

        private void IncrementSkippedFrames()
        {
            NumSkippedFrames++;
            labelSkippedFrames.Text = String.Format("{0}", NumSkippedFrames);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            int WasDPlotActive = 0;
            StopTicks = 0;
            Running = 0;

            this.Resize += new EventHandler(Form1_Resize); 
            this.pictureBox1.MouseMove += new System.Windows.Forms.MouseEventHandler(this.pictureBox1_MouseMove);
            this.pictureBox1.Paint += new System.Windows.Forms.PaintEventHandler(this.pictureBox1_Paint);

            // Initial time step = 0.1 seconds
            TimeStep = 0.1;
            textBoxTimeStep.Text = String.Format("{0}", TimeStep);
            labelMousePos.Text = "";

            // Initial plot window
            textBoxExtents.Text = "0.,-1.25,60.,1.25";

            rcPic = pictureBox1.ClientRectangle;

            dplot.DPlot_FindDPLOTLIB();
            /* We won't show error messages because doing so throws our timing off.
             * Instead we'll just keep track of how many (if any) frames we miss
             * because of timing problems or other errors. */
            dplot.DPlot_SetErrorMethod(0);

            // Start DPlot hidden
            hwndDPlot = dplot.DPlot_Start(1, ref WasDPlotActive);

            myTimer.Tick += new EventHandler(Timer_Proc);
            dxPic = this.Size.Width - pictureBox1.Size.Width;
            dyPic = this.Size.Height - pictureBox1.Size.Height;
        }
        private void Form1_Resize(object sender, System.EventArgs e)
        {
            if (this.Size.Width > dxPic && this.Size.Height > dyPic)
            {
                pictureBox1.Size = new Size(this.Size.Width - dxPic, this.Size.Height - dyPic);
                rcPic = pictureBox1.ClientRectangle;
            }
        }
        private void Timer_Proc(Object myObject,EventArgs myEventArgs)
        {
            double x, y1, y2;
            int ret;

            if (DocNum == 0) return;
            if (busy != 0)
            {
                IncrementSkippedFrames();
                return;
            }
            busy = 1;
            x = (double)(GetTickCount()-StartTicks)/1000.0;
            y1 = Math.Sin(x/4);
            y2 = Math.Cos(x/4);
   
            ret = dplot.DPlot_AddData8(DocNum, dplot.DATA_XYXY, 1, 1, ref x, ref y1);
            if(ret > 0) ret = dplot.DPlot_AddData8(DocNum, dplot.DATA_XYXY, 1, 2, ref x, ref y2);
            if(ret > 0)
            {
                labelTime.Text = String.Format("Time: {0}",x);
                if(ForceRedraw != 0)
                {
                    /* ForceRedraw gets set to 1 in response to a WM_PAINT message
                     * (when our window is covered then uncovered) */
                    dplot.DPlot_Command(DocNum, "[ViewRedraw()]");
                    ForceRedraw = 0;
                }
                /* Because we're using ManualScale, DPlot will automatically shift the
                 * plot whenever X exceeds the right extent of the plot. So our DPM
                 * structure will no longer be valid and we can't correctly translate
                 * mouse position to data space. Get an updated DPM. */
                if(x >= DPM.xhi)
                    dplot.DPlot_GetPlotMetrics(DocNum,ref DPM);
            }
            else
                IncrementSkippedFrames();
            busy = 0;
        }

        private void StopSimulation()
        {
            StopTicks = GetTickCount();
            myTimer.Stop();
            Running = 0;
            dplot.DPlot_Command(DocNum, "[FileClose()]");
            button1.Text = "&Start";
            DocNum = 0;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            /* With a very short time step, we may end up here twice in rapid succession, 
             * and user will have a hard time stopping the demo. This gives us a
             * governor: */
            if (GetTickCount() - StopTicks < 1000) return;

            // If simulation is already running, stop it
			if(Running != 0)
			{
				StopSimulation();
				return;
			}

			// Get timestep
			TimeStep = double.Parse(textBoxTimeStep.Text);

			// Change button caption to indicate that clicking it again stops the simulation
            button1.Text = "&Stop";
            labelTime.Text = "Time: 0";
            labelSkippedFrames.Text = "0";
            busy = 0;
            ForceRedraw = 0;
            NumSkippedFrames = 0;

			/* Create an empty document window in DPlot and get the document index, which
             * will be used for all subsequent DPlot_Command command strings. */
			DocNum = dplot.DPlot_Command(0, "[FileNew()]");

            string cmds = String.Format("[SetOutputWindow({0},{1},{2},{3},{4})]",
                pictureBox1.Handle,
                rcPic.Left,
                rcPic.Top,
                rcPic.Right,
                rcPic.Bottom);
            dplot.DPlot_Command(DocNum, cmds);

            /* Instruct DPLOT to allocate space for 2 curves, 1 hour's
             * worth of data: */
            cmds = String.Format("[FileArrays(2,{0})]",(int)(3600/TimeStep-0.5)+1);
			dplot.DPlot_Command(DocNum,cmds);

			/* Use thick (.03") lines to push DPlot's performance limits a bit, and
             * set a few other plot parameters. */
			dplot.DPlot_Command(DocNum,"[LineWidth(1,30)][LineWidth(2,30)]" +
				"[LineType(1,1)][LineType(2,1)]" +
				"[Color(1,255,0,0)][Color(2,0,0,255)]");

			// Set plot window and tick mark intervals
            dplot.DPlot_Command(DocNum,"[ManualScale(" + textBoxExtents.Text + ")][AutoSize()]" +
                "[TickInterval(1,5,0.25)]");

			// Labels
			dplot.DPlot_Command(DocNum,"[XAxisLabel(\"Time (seconds)\")]" +
                /* Position the legend outside the plot so that curves don't overwrite it.
                 * arguments:
                 * line length = 3/8 inches
                 * maximum rows = 1
                 * frame = 0 (no)
                 * opaque = 0 (no)
                 * x = 0.5 (center of plot)
                 * y = 0 (top of plot)
                 * number of symbols = omitted (we don't use symbols)
                 * alignment = center (1) + bottom (8) */
				"[LegendParams(0.375,1,0,0,0.5,0,,9)]" +
				"[FontPoints(1,8)][FontPoints(2,10)][FontPoints(4,10)][FontPoints(6,8)]" +
				"[Legend(1,\"SIN(t/4)\")]" +
				"[Legend(2,\"COS(t/4)\")]" +
                // Add a carriage return to title so that it doesn't overlap legend
                "[Title1(\"RealTime Demo\n\")]" +
                // Turn off anti-aliasing, because it looks pretty nasty with
                // short line segments
                "[GeneralOptions(0x00000020,0)]");

            DPM.size = Marshal.SizeOf(DPM);
            /* Initialize to a value we know is less than the minimum X extent, so 
             * that our timer routine will grab a DPLOT_PLOTMETRICS structure. 
             * This is only important for mouse position -> data space transformation 
             * handled by pictureBox1_MouseMove */
            DPM.xhi = -1.0f;

            // Get start time from API function GetTickCount
			StartTicks = GetTickCount();

			// Start the timer. See Timer_Proc for sending our values to DPlot
            Running = 1;
            myTimer.Interval = (int)(TimeStep * 1000);
            myTimer.Start();
        }
        private void pictureBox1_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            double x = e.X;
            double y = e.Y;
            double worldx, worldy;
 
            /* This check is necessary because contrary to the function name, this 
             * routine is called whenever the mouse is over the control regardless 
             * of whether it moves. If we didn't do this check then we'd update
             * labelMousePos much more often than needed. */
            if (xLast == x && yLast == y) return;
            xLast = (int)x;
            yLast = (int)y;

            if (x < DPM.hll || x > DPM.hur || y < DPM.vur || y > DPM.vll)
                labelMousePos.Text = "";
            else
            {
                /* This is only valid for linear scaling (which this demo uses).
                 * For other scaling types see the DPLOTLIB documentation. */
                worldx = DPM.xlo + (x - DPM.hll) * (DPM.xhi - DPM.xlo) / (DPM.hur - DPM.hll);
                worldy = DPM.ylo + (y - DPM.vll) * (DPM.yhi - DPM.ylo) / (DPM.vur - DPM.vll);
                labelMousePos.Text = String.Format("X={0,4:F}\nY={1,4:F}", worldx, worldy);
            }
        }
        private void pictureBox1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            // This will cause DPlot to redraw the entire plot to pictureBox1
            // at the next time step (assuming the simulation is running).
            ForceRedraw = 1;
        }
    }
}