c
c FTEST - tests DPLOTLIB.DLL with WATCOM Fortran 10.6
c
c This program has been tailored for WATCOM Fortran 10.6. Other Fortran compilers may require
c changes, particularly in the interface to DPlot_Plot.
c 
c
c Things to watch for with other compilers and general notes:
c
c 1) By default, most Fortran compilers translate the names of all functions/subroutines to UPPERCASE. 
c    The name "DPlot_Plot" in the DLL is case-specific. In other words you will most likely encounter
c    an "unresolved external" error on DPLOT_PLOT unless you take steps to tell the compiler not to
c    fold all characters to uppercase. If you are certain that you've taken the correct steps in this
c    regard and continue to get linker errors, the problem most likely lies with...
c
c 2) DPLOTLIB.LIB - The import library for DPLOTLIB.DLL. The Compaq Visual Fortran and Absoft Fortran
c    demos use the same import library as the C demo, which was produced as a byproduct of compiling
c    DPLOTLIB.DLL by the MSVC compiler. The DPLOTLIB.LIB used by the WATCOM Fortran demo was produced
c    with "wlib dplotlib +dplotlib.dll". This import library will almost certainly not work with other
c    compilers. If the import library does not work correctly, check your compiler's documentation for
c    the method used to produce import libraries from compiled DLL's.
c
c 3) This source code will work properly without changes with Open Source WATCOM Fortran 1.0. HOWEVER,
c    the supplied import library (DPLOTLIB.LIB) will NOT work with that compiler. To build FTEST with
c    Open Source WATCOM Fortran, first delete the import library DPLOTLIB.LIB, then build a new one
c    using the WATCOM library tool WLIB.EXE:
c     
c    wlib dplotlib +dplotlib.dll
c
c 4) The last parameter to DPlot_Plot is a character string containing DPlot commands. This character
c    string must be null-terminated (ends in a 0 byte). If you do NOT terminate this character string
c    with a char(0), at best DPlot will report an error in the command string; at worst DPlot and/or 
c    this demo will crash. This restriction does not apply to the character string members of the DPlot 
c    structure, although adding a terminating char(0) to those strings will not hurt anything.
c
c
c ALWAYS make sure to include this file:
c
c$include dplotapi.fi
c
c in any WATCOM FORTRAN routine that calls DPLOTLIB functions. Otherwise you'll most likely
c get 'unresolved external' errors when linking.
c
      program ftest
      implicit none
      include  'dplot.fi'

      integer option

 100  continue

      write(*,*) 'Enter 0 to exit'
      write(*,*) '      1 XY plot example'
      write(*,*) '      2 Contour plot of gridded data'
      write(*,*) '      3 Contour plot of randomly-spaced points'
      write(*,*) '      4 Box-and-whisker example'
      write(*,*) '      5 Another XY plot (30 curves)'
      write(*,*) '      6 3D scatter plot'
      write(*,*) '      7 Yet another XY plot (world pop. estimate)'
      write(*,*) '      8 Get DPlot/DPlot Jr version number'
      write(*,*) '      9 Get peak values from document 1'
      write(*,*) '     10 Surface plot with concave border'
      write(*,*) '     11 4D surface plot'
      write(*,*) '     12 4D scatter plot'
      write(*,*) '     13 Another contour plot'
      write(*,*) '     14 2D vector plot'
      write(*,*) '     15 Make movie frames of a 3D/4D plot in 3D view'
      write(*,*) '     16 Collatz Conjecture (transparent symbols)'


      write(*,*) '? '
      read(*,*) option

      select case(option)
      case(1)
        call XYExample()

      case(2)
        call ContourPlot1()

      case(3)
        call ContourPlot2()

      case(4)
        call BoxAndWhiskerPlot()

      case(5)
        call ThirtyCurves()

      case(6)
        call ThreeDScatter()

      case(7)
        call WorldPopulation()

      case(8)
        call GetVersionNumber()

      case(9)
        call GetPeakValues()

      case(10)
        call ConcaveBorderExample()

      case(11)
        call FourDSurface()

      case(12)
        call FourDScatter()

      case(13)
        call AnotherContourPlot()

      case(14)
        call VectorPlot()

      case(15)
        call MovieFrames()

      case(16)
        call Collatz()

      case(0)
        goto 9999

      end select

      goto 100
9999  continue
      end
c
c=======================================================================
c
      real*8 function random()
      implicit none
      external urand
      real     urand
      integer  seed /75347/
      random = urand(seed)
      return
      end
c
c=======================================================================
c
      subroutine XYExample()
      implicit none
      include  'dplot.fi'
c
c     Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
c     In this case we've used DATA_XYYY (one X array for one or more Y arrays).
c
c     Since the X values are evenly spaced we could also use
c     DATA_DXY  - X has only 2 elements, DX and X0
c
c     And you can ALWAYS use
c     DATA_XYXY - Each curve uses its own X array. This is the only
c                 option available if the curves have different X values or
c                 a different number of points.

      integer*4 NP
      parameter (NP=1001)
      record /DPLOT/ DPlot
      integer*4 i
      integer*4 ret
      real*8    x(NP), y(2*NP)
      real*8    PI
      ret = 1
      PI = 4.*atan(1.0)
      do i=1,NP
        x(i)   = (4.*i)/(NP-1)
        y(i)   = sin(PI*x(i))
        y(i+NP)= cos(PI*x(i))
      end do

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_XYYY
      DPlot.MaxCurves   = 2
      DPlot.MaxPoints   = NP
      DPlot.NumCurves   = 2
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.LegendX     = 0.05
      DPlot.LegendY     = 0.05
      DPlot.NP(1)       = NP
      DPlot.NP(2)       = NP
      DPlot.LineType(1) = LINESTYLE_SOLID
      DPlot.LineType(2) = LINESTYLE_LONGDASH
      DPlot.Legend(1)   = 'sin({\sp}x)'
c                              {\s is DPlot-speak for "use symbol font"
      DPlot.Legend(2)   = 'cos({\sp}x)'  
      DPlot.Title(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 
c		
c     The command string can be as complex as you like, limited to 32K characters.
c     As an alternative, if your program will be producing many similar plots then
c     you might prefer to create a preferences file, edit the file with a text 
c     editor to remove unwanted entries, and use the [GetPreferences("filename")]
c     command. 
c
      ret = DPlot_Plot8(DPlot,x,y,
     1                 '[ManualScale(0,-1.25,4,1.25)]'//
     2                 '[TickInterval(1,0.5,0.25)]'//
     3                 '[Caption("DPLOTLIB XY Test")]'//
     4                 '[DocMaximize()]'//
     5                 '[ClearEditFlag()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine ContourPlot1()
      implicit none
      include  'dplot.fi'
c
c     Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y)
c
      integer*4 NX, NY
      parameter (NX=61, NY=41)
      record  /DPLOT/ DPlot
      integer*4 i, j, ret
      real*8    extents(4)
      real*8    xx, yy
      real*8    z(NX*NY)

c     extents are, in order, xlo, ylo, xhi, yhi
      extents(1) = -3.
      extents(2) = -2.
      extents(3) =  3.
      extents(4) =  2.
      do i=1,Nx
        xx = extents(1) + ((extents(3)-extents(1))*i)/(Nx-1)
        do j=1,Ny
          yy = extents(2) + ((extents(4)-extents(2))*j)/(Ny-1)
          z((i-1)*Ny+j) = sin(xx)*cos(yy)
        end do
      end do

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_3D
c
c     MaxCurves and MaxPoints members are used for the size of the grid, in grid cells
c
      DPlot.MaxCurves   = Nx-1
      DPlot.MaxPoints   = Ny-1
      DPlot.NumCurves   = 1
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPlot.Title(2)    = 'z = sin(x)*cos(y)'
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 
		
      ret = DPlot_Plot8(DPlot,extents,z,
     1       '[Caption("DPLOTLIB Contour Test 1")]'//
     2       '[Contour3D(1)][ContourGrid(1)][ContourAxes(1)]'//
     3       '[ContourView(30,20)][ContourLevels(20,-1,1)]'//
     4       '[ContourScales(1,1,1)]'//
     5       '[ContourLighting(2,0.2)]'//                     ! Phong shading, 20% ambient light
     6       '[FontPoints(1,10)][FontPoints(6,10)]'//
     7       '[FontPoints(4,12)][FontPoints(5,12)][FontPoints(8,12)]'//
     8       '[ZAxisLabel("z")][DocMaximize()]'//char(0))
      return
	  end
c
c======================================================================
c
      subroutine ContourPlot2()
      implicit none
      include  'dplot.fi'
c
c     Contour plot with randomly spaced points
c
      structure /XYZ/
        real*8 x
        real*8 y
        real*8 z
      end structure

      integer*4 i, ret
      real*8    PI
      real*8    dummy

      integer*4 NZ
      parameter (NZ=1000)
      record  /XYZ/ Node(NZ)
      record  /DPLOT/ DPlot

      real*8   random
      external random

      PI = 4.*atan(1.0)
      do i=1,Nz
         Node(i).x = -PI + 2*PI*random()
         Node(i).y = -PI + 2*PI*random()
         Node(i).z = sin(Node(i).x)*cos(Node(i).y)
      end do

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_3DR
      DPlot.MaxCurves   = 1
      DPlot.MaxPoints   = Nz
      DPlot.NumCurves   = 1
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPlot.Title(2)    = 'z = sin(x)*cos(y)'
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 
		
      ret = DPlot_Plot8(DPlot,dummy,Node,
     1           '[Caption("DPLOTLIB Contour Test 2")]'//
     2           '[Contour3D(0)][ContourGrid(0)]'//
     3           '[ContourLevels(21,-1,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine BoxAndWhiskerPlot()
      implicit none
      include  'dplot.fi'
c
c     Box-and-whisker plot - 3 groups
c
      integer*4 i, j, k, ret
      real*8    dummy

      integer*4 NP
      parameter (NP=150)
      record  /DPLOT/ DPlot
      real*8  y(NP)

      real*8   random
      external random

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_1D
      DPlot.MaxCurves   = 3
      DPlot.MaxPoints   = 50
      DPlot.NumCurves   = 3
      DPlot.LegendX		= 0.02
      DPlot.LegendY		= 0.7
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'Data sent to DPlot via DPLOTLIB.DLL'
      DPlot.XAxis       = ' '
      DPlot.YAxis       = 'Amplitude'

      do i=1,DPlot.NumCurves
        DPlot.NP(i) = DPlot.MaxPoints
        DPlot.Legend(i) = 'Group '//char(64+i)//char(0)
      end do

c     Generate group data, centered around 5.0 with progressively larger variations
      k = 1
      do i=1,DPlot.NumCurves
        do j=1,DPlot.NP(i)
          y(k) = 5.0 + (i+1)*(-0.5+random())
          k = k+1
        end do
      end do
		
      ret = DPlot_Plot8(DPlot,dummy,y,
     1           '[Caption("DPLOTLIB Box-and-whisker Test")]'//
     2           '[Stat_PlotType(0)][Stat_GroupInfo(15,4)]'//
     3           '[Stat_BWOptions(1,1,0,1,1.5,2,3)]'//
     4           '[Stat_GrandMean(3,30)]'//
     5           '[FontPoints(1,10)][FontPoints(6,10)]'//
     6           '[DocMaximize()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine ThirtyCurves()
      implicit none
      include 'dplot.fi'

      record /DPLOT/ DPlot
      integer NC, NP
      parameter (NC=30, NP=1001)
      real*8 x(NP), y(NP*NC)
      integer i, j, ret
      real*8  PI
      parameter (PI=3.1415926535897932384626433832795)

c     Simulate a square wave with a slew of sine waves. First 29 curves are
c     sine waves, last curve is the addition of 1st 29.

      do i=0,NP-1
        x(i+1) = (2.*PI*i)/(NP-1)
        y(i+(NC-1)*NP+1) = 0.0
        do j=0,NC-2
          y(i+j*NP+1) = sin((j*2+1)*x(i+1))/(j*2+1)
          y(i+(NC-1)*NP+1) = y(i+(NC-1)*NP+1) + y(i+j*NP+1)
        end do
        y(i+(NC-1)*NP+1) = y(i+(NC-1)*NP+1) * 4.0/PI
      end do

      DPlot.Version		= DPLOT_DDE_VERSION
      DPlot.hwnd			= 0
      DPlot.DataFormat	= DATA_XYYY
      DPlot.MaxCurves		= NC
      DPlot.MaxPoints		= NP
      DPlot.NumCurves		= NC
      DPlot.Scale			= SCALE_LINEARX_LINEARY
      do j=1,NC
        DPlot.NP(j) = NP
        DPlot.LineType(j) = LINESTYLE_SOLID
        if(j .lt. NC) write(DPlot.Legend(j),'(A,I2,A,I2)')
     +     'sin(',j*2-1,'x)/',j*2-1
      end do
      DPlot.Legend(NC) = '4/{\sp}(sin(x)+sin(3x)/3+sin(5x)/5+...)'
      DPlot.Title(1) = 'Square wave approximation'
      DPlot.XAxis    = 'x'
      DPlot.YAxis    = 'y'

      ret = DPlot_Plot8(DPlot,x,y,
     +       '[ManualScale(0,-1.25,6.5,1.25)]'//
     +       '[TickInterval(1,0.5,0.25)]'//
     +       '[LegendParams(0.25,10,1,1,0,1,0,8)]'//
     +       '[LineWidth(-2,0)][Caption("DPLOTLIB XY Test")]'//
     +       '[DocMaximize()][ClearEditFlag()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine ThreeDScatter()
      implicit none
      include  'dplot.fi'
c
c     3D scatter plot
c
      structure /XYZ/
        real*8 x
        real*8 y
        real*8 z
      end structure

      integer*4 i, j, k
      integer*4 DocNum
      real*8    xmin, xmax, ymin, ymax, zmin, zmax
      real*8    xcenter, ycenter, zcenter
      real*8    phi, theta, theta_inc, radius
      real*8    dummy
      real*8    PI
      character szCommands*1024
      record /XYZ/ Node(3000)
      record /DPLOT/ DPlot

      real*8   random
      external random

      PI = 4.*atan(1.0)

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_3DS
      DPlot.MaxCurves   = 3
      DPlot.MaxPoints   = 1500
      DPlot.NumCurves   = 3
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'DPLOTLIB 3D Scatter Plot'
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 

      DPlot.NP(1) = 500
      DPlot.NP(2) = 1000
      DPlot.NP(3) = 1500
c
c     Keep track of the minimum and maximum X,Y, and Z values so we can
c     force plot extents to good values, "good" in this case being
c     whole numbers.
c
      xmin = xmax = 0
      ymin = ymax = 0
      zmin = zmax = 0

      k = 1       ! index into Scatter array
      do i = 1, DPlot.NumCurves
c
c       Center of each sphere is at some random location between
c       (-5,-5,-5) and (5,5,5).
c
        xcenter = -5.0 + 10.0*random()
        ycenter = -5.0 + 10.0*random()
        zcenter = -5.0 + 10.0*random()
c
c       radii are proportional to number of points
c
        radius = 0.005 * DPlot.NP(i)
c
        xmin = dmin1(xmin,xcenter-radius)
        xmax = dmax1(xmax,xcenter+radius)
        ymin = dmin1(ymin,ycenter-radius)
        ymax = dmax1(ymax,ycenter+radius)
        zmin = dmin1(zmin,zcenter-radius)
        zmax = dmax1(zmax,zcenter+radius)
c
c       increment in theta (angle around z axis) gives roughly
c       equal spacing between points in all directions:
c
        theta_inc = 2.0 * sqrt(dble(DPlot.NP(i)))
        do j = 1, DPlot.NP(i)
          phi = -PI / 2 + PI * dble(j-1) / dble(DPlot.NP(i))
          theta = theta_inc * PI * dble(j-1) / dble(DPlot.NP(i))
          Node(k).x = xcenter + (radius * cos(theta) * cos(phi))
          Node(k).y = ycenter + (radius * sin(theta) * cos(phi))
          Node(k).z = zcenter + (radius * sin(phi))
          k = k + 1
        end do
      end do

      write(szCommands,'(A,I3,1H,,I3,1H,,I3,1H,,I3,1H,,I3,1H,,I3,A)')
     1           '[Caption("DPLOTLIB 3D Scatter Plot")]'//
     2           '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     3           '[TickInterval(1,1,1,1)][ContourAxes(1)]'//
     4           '[ColorBasis(1)][ContourColorScheme(2)]'//
     5           '[ContourCustomColors(255,65280,16711680)]'//
     6           '[ContourScales(1,1,1)][SymbolOutline(1)]'//
     7           '[ZAxisLabel("z")]'//
     8           '[ManualScale(',
     9           int(xmin-0.5),int(ymin-0.5),
     A           int(xmax+0.5),int(ymax+0.5),
     B           int(zmin-0.5),int(zmax+0.5),
     C           ')][FontPoints(1,10)][FontPoints(6,10)]'//
     D           '[DocMaximize()]'//char(0)
      DocNum = DPlot_Plot8(DPlot,dummy,Node,szCommands)
      return
      end
c
c======================================================================
c
      integer function ExcelDay(Month,Day,Year)
      implicit none
      integer*2 Month, Day, Year
      
	if(Year .lt. 1900 .or. Year .gt. 2078) then
        ExcelDay = -1
      else
        if(Month .gt. 2) then
          ExcelDay = int(365.25*Year)+int(30.6001*(Month+1))+Day-694037
        else
          ExcelDay = int(365.25*(Year-1))+int(30.6001*(Month+13))+Day-
     +                694037
        endif
      endif
      return
      end
c
c======================================================================
c
      subroutine WorldPopulation()
      implicit none
      include 'dplot.fi'
c
c     Estimate of world population
c
      record /DPLOT/ DPlot
      integer NUMDAYS, SAMPLESPERDAY
      parameter (NUMDAYS=10,SAMPLESPERDAY=24*60*6)    ! 10 second intervals
      real*8 REPLACEMENTRATE
      parameter (REPLACEMENTRATE=0.013)

      integer*2 Year, Month, Day
      integer*2 Hrs, Mins, Secs, HSecs
      integer NP
      parameter (NP=NUMDAYS*SAMPLESPERDAY+1)
      real*8  JanOne04Pop
      parameter (JanOne04Pop = 6393657063.)
      real*8  BaseDay
      real*8  CurrentDate
      real*8  elapsed_time
      real*8  x(NP), y(NP)
      integer i, ret

      integer  ExcelDay
      external ExcelDay

      BaseDay = ExcelDay(1,1,2004)
c
c     GETDAT and GETTIM are specific to WATCOM FORTAN
c
      call GETDAT(Year,Month,Day)
      call GETTIM(Hrs,Mins,Secs,HSecs)
      CurrentDate = ExcelDay(Month,Day,Year) +
     +         dble(Hrs)/24 + dble(Mins)/(24*60)

c     We'll estimate the world population every 30 seconds for NUMDAYS days
      
      do i=1,NP
        x(i) = CurrentDate + dble(i-1)/dble(SAMPLESPERDAY)
        elapsed_time = x(i) - BaseDay
        y(i) = dint( JanOne04Pop * 
     +          exp(REPLACEMENTRATE*elapsed_time/365.25))
      end do

      DPlot.Version		= DPLOT_DDE_VERSION
      DPlot.hwnd          = 0
      DPlot.DataFormat	= DATA_XYXY
      DPlot.MaxCurves		= 1
      DPlot.MaxPoints		= NP
      DPlot.NumCurves		= 1
      DPlot.Scale			= SCALE_LINEARX_LINEARY
      DPlot.NP(1)			= NP
      DPlot.LineType(1)	= LINESTYLE_SOLID
      DPlot.Title(1) = 'World Population Estimate'
      write(DPlot.Title(2),'(A,F11.0)')
     +  'based on Jan 1, 2004 population of ',JanOne04Pop
      write(DPlot.Title(3),'(A,F4.1,1H%)') 'and a replacement rate of ',
     +        100*REPLACEMENTRATE
      DPlot.XAxis = 'Date'
      DPlot.YAxis = 'Population'

      ret = DPlot_Plot8(DPlot,x,y,
     +       '[Caption("DPLOTLIB XY Test")]'//
     +       '[DateFormat("M/dd/yy")]'//
     +       '[TimeFormat("h:mm:ss")]'//
     +       '[NumberFormat(0,6)][NumberFormat(1,11)]'//
     +       '[ClearEditFlag()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine GetVersionNumber()
      implicit none
      include  'dplot.fi'

      integer result
      character version*32

      result = DPlot_GetVersion(version)
      if(result .eq. 0) then
        write(*,*) 'DPlot_GetVersion failed!'
      elseif(result .eq. 1) then
        write(*,*) 'DPlot Version ',version
      else
        write(*,*) 'DPlot Jr Version ',version
      endif
      return
      end
c
c======================================================================
c
      subroutine GetPeakValues()
      implicit none
      include  'dplot.fi'

      character szPeaks*1024
      integer   dwSize
      integer   ret

      dwSize = len(szPeaks)
      ret = DPlot_Request(1,'ListPeaks'//char(0),szPeaks,dwSize)
      if(ret .gt. 0) then
        dwSize=index(szPeaks,char(0))-1
        write(*,*) szPeaks(1:dwSize)
      elseif(ret .eq. -3) then
        write(*,*) 'Insufficient buffer size, ',dwSize,' required.'
      else
        write(*,*) 'Return value of DPlot_Request=',ret
      endif
      return
      end
c
c======================================================================
c
      subroutine ConcaveBorderExample()
      implicit none
      include  'dplot.fi'
c
c     Surface plot with concave border
c
      structure /XYZ/
        real*8 x
        real*8 y
        real*8 z
      end structure

      structure /XYPT/
        real*8 x
        real*8 y
      end structure

      integer*4 i, DocNum, ret
      real*8    x0, y0, side
      real*8    dummy
      real*8    PI
      record /XYZ/ Node(10)
      record /XYPT/ Border(10)
      record /DPLOT/ DPlot


      PI = 4.*atan(1.0)

      x0 = 0.             ! center of star/pentagon
      y0 = 0.
      side = 1.0          ! length of each pentagon side

      Node(1).x = x0 - 0.5*side
      Node(1).y = y0 - 0.5*side*tan(54*PI/180)
      Node(1).z = 0.0
      Node(2).x = x0
      Node(2).y = Node(1).y+0.5*side*tan(36*PI/180)
      Node(2).z = 1.0
      Node(3).x = x0 + 0.5*side
      Node(3).y = Node(1).y
      Node(3).z = 0.0
      Node(4).x = Node(3).x-0.5*side*(sin(18*PI/180)/cos(36*PI/180))
      Node(4).y = Node(3).y+0.5*side*(cos(18*PI/180)/cos(36*PI/180))
      Node(4).z = 1.0
      Node(5).x = Node(3).x+side*sin(18*PI/180)
      Node(5).y = Node(3).y+side*cos(18*PI/180)
      Node(5).z = 0.0
      Node(6).x = x0+0.5*side*(sin(18*PI/180)/cos(36*PI/180))
      Node(6).y = Node(5).y
      Node(6).z = 1.0
      Node(7).x = x0
      Node(7).y = Node(5).y+side*sin(36*PI/180)
      Node(7).z = 0.0
      Node(8).x = 2*x0 - Node(6).x
      Node(8).y = Node(6).y
      Node(8).z = 1.0
      Node(9).x = 2*x0 - Node(5).x
      Node(9).y = Node(5).y
      Node(9).z = 0.0
      Node(10).x = 2*x0 - Node(4).x
      Node(10).y = Node(4).y
      Node(10).z = 1.0

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_3DR
      DPlot.MaxCurves   = 1
      DPlot.MaxPoints   = 10
      DPlot.NumCurves   = 1
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'Surface Plot with Concave Edges'
      DPlot.Title(2)    = 'See {\iDefine Boundary} command '//
     1                    'on {\iOptions} menu'
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 
		
      DocNum = DPlot_Plot8(DPlot,dummy,Node,
     1           '[Caption("DPLOTLIB Surface Plot w/ Concave Edges")]'//
     2           '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     3           '[ContourLevels(20,0,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0))
c
c     At this point surface is convex - a pentagon. Add border
c     points, which in this example are simply a repeat of the x,y points
c     of the surface
c
      if(DocNum .gt. 0) then
        do i=1,10
          border(i).x = Node(i).x
          border(i).y = Node(i).y
        end do
        ret = DPlot_3DBorder(DocNum,10,border)
      endif
      return
      end
c
c=======================================================================
c
      integer function floor(x)
      implicit none
      real*8 x

      if(x .ge. 0) then
        floor = int(x)
      elseif(dmod(x,1.D0) .eq. 0.0) then
        floor = int(x)
      else
        floor = int(x)-1
      endif
      return
      end
c
c=======================================================================
c
      integer function ceil(x)
      implicit none
      real*8 x

      if(x .le. 0) then
        ceil = int(x)
      elseif(dmod(x,1.D0) .eq. 0.0) then
        ceil = int(x)
      else
        ceil = int(x)+1
      endif
      return
      end
c
c=======================================================================
c
      subroutine FourDSurface()
      implicit none
      include 'dplot.fi'
c
c     4D surface plot on a rectangular grid
c
      record /DPLOT/ DPlot
      integer Nx, Ny            ! Rows and columns
      parameter (Nx=121, Ny=121)
      integer i, j, k, ret
      real*8  x, y, z
      real*8  dummy
      structure /XYZA/
        real*8 x
        real*8 y
        real*8 z
        real*8 a
      end structure
      record /XYZA/ Node(Nx*Ny)

      k = 1
      do i = 0,Nx-1
        x = 5.0 + 30.*dble(i)/dble(Nx-1)
        do j = 0,Ny-1
          Node(k).x = x
          Node(k).y = y = 5.0 + 30.*dble(j)/dble(Ny-1)
          if(x .eq. 20. .and. y .eq. 20.) then
            z = sin(1.0)
          else
            z = sin(sqrt((x-20.)*(x-20.)+(y-20.)*(y-20)))/
     +          sqrt((x-20.)*(x-20.)+(y-20.)*(y-20.))
          endif
          z = (z + 1.5)*100.
          Node(k).z = z
          Node(k).a = 10. * (1.1 + sin((x-20.)/5.)*cos((y-20.)/10.))
          k = k+1
        end do
      end do

      DPlot.Version		= DPLOT_DDE_VERSION
      DPlot.DataFormat	= DATA_4D
      DPlot.MaxCurves		= 1
      DPlot.MaxPoints		= Nx*Ny
      DPlot.NumCurves		= 1
      DPlot.Scale			= SCALE_LINEARX_LINEARY
      DPlot.Title(1) = 'Data sent to DPlot via DPLOTLIB.DLL'
      DPlot.Title(2) = 'z = 100 * (sin(sqrt((x-20.){\u2}+'//
     +                                     '(y-20.){\u2})) / '//
     +                      'sqrt((x-20.){\u2}+(y-20.){\u2}) + 1.5)'
      DPlot.Title(3) = 'a = 10. * (1.1 + sin((x-20.)/5.)*'//
     +                 'cos((y-20.)/10.))'
      DPlot.XAxis    = 'x'
      DPlot.YAxis    = 'y'
	
      ret = DPlot_Plot8(DPlot,dummy,Node,
     +   '[Caption("DPLOTLIB 4D Surface Plot Test")]'//
     +   '[Contour3D(1)][ContourMethod(0)][ContourGrid(0)]'//
     +   '[ContourAxes(1)][ContourView(118,28)]'//
     +   '[ContourLevels(22,0,22)][ContourScales(1,1,0.2)]'//
     +   '[ContourLighting(0,0.05)]'//
     +   '[ContourColorScheme(2)]'//
     +   '[ContourCustomColors(0,0x00FF047D,0x000B1EB3,0x000068DC,'//
     +   '0x0000FAFE)]'//
     +   '[FontPoints(1,10)][FontPoints(4,14)][FontPoints(5,14)]'//
     +   '[FontPoints(6,10)][FontPoints(8,14)]'//
     +   '[ZAxisLabel("z")][AAxisLabel("Amplitude")][DocMaximize()]'//
     +   char(0))
      return
      end
c
c=======================================================================
c
      subroutine FourDScatter()
      implicit none
      include 'dplot.fi'
c
c     4D scatter plot - random points on 3 spheres.
c
      record /DPLOT/ DPlot
      integer NC, Nz
      parameter (NC=3, Nz=1000)
c
c     Nz is the number of points in each of the NC data sets. Data sets CAN have
c     differing numbers of points. In either case, the Node array is packed with 
c     no unused nodes. For example the first point in the 2nd data set should
c     immediately follow the last point in the 1st data set.
c
      integer i, j, ret
      real*8 zz, radius, theta, phi
      real*8 theta_inc
      real*8 xmin, xmax, ymin, ymax, zmin, zmax
      real*8 xx, yy
      real*8 dummy
      real*8  PI
      parameter (PI=3.1415926535897932384626433832795)
      structure /XYZA/
        real*8 x
        real*8 y
        real*8 z
        real*8 a
      end structure
      record /XYZA/ Node(NC*Nz)
      character cmd*512

      integer  ceil, floor
      external ceil, floor
      real*8   random
      external random

      xmin = xmax = 0
      ymin = ymax = 0
      theta_inc = 2.0*sqrt(dble(Nz))
      do j=0,NC-1
        xx = -10.+20.*random()         ! Center of sphere
        yy = -10.+20.*random()
        zz = -10.+20.*random()
        radius = 4.+4.*random()
        xmin = dmin1(xmin,xx-radius)
        xmax = dmax1(xmax,xx+radius)
        ymin = dmin1(ymin,yy-radius)
        ymax = dmax1(ymax,yy+radius)
        zmin = dmin1(zmin,zz-radius)
        zmax = dmax1(zmax,zz+radius)
        do i=j*Nz,(j+1)*Nz-1
          phi = -PI/2+PI*dble(i-j*Nz)/dble(Nz)
          theta = theta_inc*PI*dble(i-j*Nz)/dble(Nz)
          Node(i+1).x = xx + radius*cos(theta)*cos(phi)
          Node(i+1).y = yy + radius*sin(theta)*cos(phi)
          Node(i+1).z = zz + radius*sin(phi)
          Node(i+1).a = Node(i+1).x               ! No reason, but it has to be something!
        end do
        write(DPlot.Legend(j+1),'(A,I1)') 'Sphere ',j+1
        DPlot.NP(j+1) = Nz
      end do

      DPlot.Version           = DPLOT_DDE_VERSION
      DPlot.hwnd              = 0
      DPlot.DataFormat        = DATA_4DS
      DPlot.MaxCurves         = NC
      DPlot.MaxPoints         = Nz
      DPlot.NumCurves         = NC
      DPlot.Scale             = SCALE_LINEARX_LINEARY
      DPlot.Legend(0)         = 'Legend'
      DPlot.Title(1)          = 'DPLOTLIB - Points on Spheres'
      DPlot.Title(2)          = 'Amplitude=X'
      DPlot.XAxis             = 'x'
      DPlot.YAxis             = 'y'

      xmin = floor(xmin)
      xmax = ceil(xmax)
      ymin = floor(ymin)
      ymax = ceil(ymax)
      zmin = floor(zmin)
      zmax = ceil(zmax)

      write(cmd,'(A,5(F4.0,1H,),F4.0,A,A)')
     +  '[Caption("DPLOTLIB Test 12")]'//
     +  '[TickInterval(1,1,1,1)]'//
     +  '[ManualScale(',xmin,ymin,xmax,ymax,zmin,zmax,
     +  ')][ColorBasis(0)][ContourColorScheme(2)]'//
     +  '[ContourCustomColors(255,65535,16711680)]'//
     +  '[ContourScales(1,1,1)]'//
     +  '[FontPoints(1,10)][FontPoints(6,10)]'//
     +  '[DocMaximize()][SymbolType(-1,288)]'//
     +  '[SymbolSize(1,100)][SymbolSize(2,150)][SymbolSize(3,200)]'//
     +  '[SymbolOutline(1)]',
     +  char(0)

      ret = DPlot_Plot8(DPlot,dummy,Node,cmd)
      return
      end
c
c=======================================================================
c
      subroutine AnotherContourPlot()
      implicit none
      include 'dplot.fi'

      record /DPLOT/ DPlot

      integer Nx, Ny
      parameter (Nx=361,Ny=161)
      integer i, j, ret
      real*8  extents(4)
      real*8  xx, yy
      real*8  z(Nx*Ny)
      real*8  Vc, Vs, Vnet, Vf
      real*8  theta
      real*8  DEGTORAD
      DEGTORAD(xx) = (xx/57.295779513082320876798154814105)

      extents(1) = 0.0       ! xlo
      extents(2) = 0.0       ! ylo
      extents(3) = 360.0     ! xhi
      extents(4) = 1.0       ! yhi
      do i=1,Nx
        xx = extents(1) + ((extents(3)-extents(1))*(i-1))/(Nx-1)
        do j=1,Ny
          yy = extents(2) + ((extents(4)-extents(2))*(j-1))/(Ny-1)
          Vc = 1.0-yy*cos(DEGTORAD(xx))
          Vs = yy*sin(DEGTORAD(xx))
          Vnet = sqrt(Vc*Vc + Vs*Vs)
          theta = atan2(Vs*Vs,Vc*Vc)
          Vf = Vnet*Vnet * dabs(cos(theta))
          if(Vc .lt. 0) then
            z((i-1)*Ny+j) = -Vf
          else
            z((i-1)*Ny+j) = Vf
          endif
        end do
      end do
      DPlot.Version         = DPLOT_DDE_VERSION
      DPlot.hwnd            = 0
      DPlot.DataFormat      = DATA_3D
      DPlot.MaxCurves       = Nx-1               ! Size of the grid, in grid cells
      DPlot.MaxPoints       = Ny-1
      DPlot.NumCurves       = 1
      DPlot.Scale           = SCALE_LINEARX_LINEARY
      DPlot.Title(1) = 'Relative Drag Force for a Spherical Object'
      DPlot.Title(2) = 'Drag{\dactual}/Drag{\dno wind}\n'
      DPlot.XAxis    = 'Wind Direction, degrees'
      DPlot.YAxis    = 'Wind Speed/Ground Speed'

      ret = DPlot_Plot8(DPlot,extents,z,
     + '[Caption("DPLOTLIB Contour Test 13")]'//
     + '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     + '[ContourAxes(1)][ContourLevels(40,0,4)]'//
     + '[ContourScales(1,360,1)][ContourLighting(2,0.05)]'//
     + '[ManualScale(0,0,360,1,0,4)][TickInterval(1,10,0.1,1)]'//
     + '[FontPoints(1,10)][FontPoints(6,10)][ZAxisLabel("z")]'//
     + '[XYLabel(0,1,0,"Tailwind")][XYLabel(180,1,0,"Headwind")]'//
     + '[XYLabel(360,1,0,"Tailwind")][DocMaximize()][ClearEditFlag()]'//
     +  char(0))

      return
      end
c
c=======================================================================
c
      subroutine VectorPlot()
      implicit none
      include  'dplot.fi'

      integer Nx, Ny, NP
      parameter (Nx=37, Ny=16, NP=Nx*Ny)  ! Columns, rows, total points/curve
      record /DPLOT/ DPlot
      integer i, j, k, ret
      real*8  xlo, ylo, xhi, yhi
      real*8  xx, yy
      real*8  x(NP), y(3*NP)
      real*8  alpha
      real*8  PI
      parameter (PI=3.1415926535897932384626433832795)
      real*8  DEGTORAD
      DEGTORAD(alpha) = (alpha/57.295779513082320876798154814105)

      xlo = 0.0
      ylo = 0.0
      xhi = 360.0
      yhi = 1.5
      k = 1
      do i=1,Nx
        xx = xlo + ((xhi-xlo)*(i-1))/(Nx-1)
        do j = 1,Ny
          yy = ylo + ((yhi-ylo)*(j-1))/(Ny-1)
          x(k) = xx
          y(k) = yy
          alpha = xx
          if(alpha .lt. 0) alpha = alpha+360
          y(k+NP) = PI-atan2(yy*sin(DEGTORAD(alpha)),
     +                       1.0-yy*cos(DEGTORAD(alpha)))
          if(y(k+NP) .lt. 0.0) y(k+NP) = y(k+NP) + 2.*PI
          if(y(k+NP) .ge. 2.*PI) y(k+NP) = y(k+NP) - 2.*PI
          y(k+2*NP) = sqrt(1.0-2*yy*cos(DEGTORAD(alpha))+yy*yy)
          k = k+1
        end do
      end do

      DPlot.Version       = DPLOT_DDE_VERSION
      DPlot.hwnd          = 0
      DPlot.DataFormat    = DATA_XYYY
      DPlot.MaxCurves     = 3
      DPlot.MaxPoints     = NP
      DPlot.NumCurves     = 3
      DPlot.Scale         = SCALE_LINEARX_LINEARY
      DPlot.NP(1) = DPlot.NP(2) = DPlot.NP(3) = NP
      DPlot.LineType(1)   = 
     +DPlot.LineType(2)   =
     +DPlot.LineType(3)   = LINESTYLE_NONE
      DPlot.SymbolType(1) = 
     +DPlot.SymbolType(2) =
     +DPlot.SymbolType(3) = SYMBOLSTYLE_DOT
      DPlot.Title(1) = 'Air Speed/Ground Speed\n'   ! Add a linefeed to make room for labels at the top
      DPlot.XAxis    = 'Wind Direction, degrees'
      DPlot.YAxis    = 'Wind Speed/Ground Speed'

      ret = DPlot_Plot8(DPlot,x,y,
     + '[Caption("DPLOTLIB Vector Plot Test")]'//
     + '[ManualScale(-10,-0.1,370,1.6)]'//
     + '[TickInterval(1,10,0.1)]'//
     + '[FontPoints(1,10)][FontPoints(6,10)][AxisType(2)]'//
     + '[XYLabel(0,1,0,"Tailwind")][XYLabel(90,1,0,"Left")]'//
     + '[XYLabel(180,1,0,"Headwind")][XYLabel(270,1,0,"Right")]'//
     + '[XYLabel(360,1,0,"Tailwind")]'//
     + '[VectorPlot(1,2,3,0x0000FF00,0x000000FF,100,500)]'//
     + '[VectorPlotOptions(6,150,20)]'//
     + '[DocMaximize()][ClearEditFlag()]'//char(0))

      return
      end
c
c=======================================================================
c
      subroutine MovieFrames()
      implicit none
      include  'dplot.fi'
c
c     Make movie frames of current 3D plot
c
c     NOTE: There's currently (2.2.4.1) no way to check for whether a 3D/4D plot is
c           displayed in a 3D view. This routine will work with 2D views w/o a problem, but
c           all frames will be identical.
c
      character szCmd*128
      character szValue*32
      integer   dwSize
      integer   az_increment
      integer   width, height
      integer   i, L, ret
      character szDestination*256
      integer   DocNum, DocType

      dwSize = len(szValue)
      if(DPlot_Request(0,'ActiveDocument'//char(0),szValue,dwSize)
     +               .lt. 0) then
        write(*,*) 'Error retrieving active document index from DPlot.'
        return
      endif
      L = index(szValue,char(0))
      if(L .ne. 0) szValue(L:L)=' '
      read(szValue,*) DocNum
      if(DocNum .le. 0) then
        write(*,*) 'No document is open. To use this feature you '//
     +          'need an open 3D or 4D plot with a 3D view in DPlot.'
        return
      endif

      dwSize = len(szValue)
      if(DPlot_Request(DocNum,'DocumentType'//char(0),szValue,dwSize)
     +      .lt. 0) then
        write(*,*) 'Error retrieving document type (random 3D '//
     +             'points, etc.) from document index ',DocNum
        return
      endif
      L = index(szValue,char(0))
      if(L .ne. 0) szValue(L:L)=' '
      read(szValue,*) DocType
      if((DocType .ne. DATA_3D)  .and.    ! Z values on a rectangular grid
     +   (DocType .ne. DATA_3DR) .and.    ! Random X,Y,Z values
     +   (DocType .ne. DATA_3DS) .and.    ! 3D scatter plot
     +   (DocType .ne. DATA_4D ) .and.    ! 4D surface plot
     +   (DocType .ne. DATA_4DS)      ) then
        write(*,*) 'This feature only works with 3D/4D documents.'
        write(*,*) 'The currently active document is type ',DocType
        return
      endif
      write(*,*) 'Enter azimuth increment in degrees (0 to abort)... '
      read(*,*) az_increment
      if(az_increment .eq. 0) return
      write(*,*) 'Enter desired bitmap width and height, in pixels '//
     +           '(640x480 suggested)... '
      read(*,*) width, height
      write(*,*) 'Enter destination path for bitmap images... '
      read(*,'(A)') szDestination
      L = lentrim(szDestination)
      if(szDestination(L:L) .eq. '\') then
        szDestination(L:L) = ' '
        L = L-1
      endif

      write(szCmd,'(A,I4,1H,,I4,A,I4,1H,,I4,A,A)')
     +          '[SetClientRect(1,0,0,',width-1,height-1,
     +          ')][SetPluginImageDims(',width,height,
     +          ',24,0,0)][SetImageCrop(0)][Preserve3DScaleFactor()]',
     +          char(0)
      ret = DPlot_Command(DocNum,szCmd)
      do i=0,359,az_increment
        write(szCmd,'(A,I3,A)') '[ContourView(',i,')]'//char(0)
        ret = DPlot_Command(DocNum,szCmd)
        write(szCmd,'(A,A,1H\,I3.3,A)')
     +         '[FileSaveAs("BMP Picture","',
     +         szDestination(1:L),i,'.bmp")]'//char(0)
        ret = DPlot_Command(DocNum,szCmd)
        write(*,*) i
      end do
      ret = DPlot_Command(DocNum,'[Preserve3DScaleFactor(0)]'//char(0))
      return
      end
c
c=======================================================================
c
      subroutine Collatz()
      implicit none
      include  'dplot.fi'
c
c     Collatz Conjecture with transparent symbols.

      integer*4 NP
      parameter (NP=100000)
      record /DPLOT/ DPlot
      integer*4 i, j, k
      integer*4 ret
      real*8    x(NP), y(NP)
      ret = 1

      do i=1,NP
        j = 0
        k = i
        do while(k .ne. 1)
           if(mod(k,2) .eq. 0) then
             k = k/2
           else
             k = k*3 + 1
           endif
           j = j+1
        end do
        x(i) = dble(i)
        y(i) = dble(j) 
      end do


      DPlot.Version       = DPLOT_DDE_VERSION
      DPlot.hwnd          = 0
      DPlot.DataFormat    = DATA_XYXY
      DPlot.MaxCurves     = 1
      DPlot.MaxPoints     = NP
      DPlot.NumCurves     = 1
      DPlot.Scale         = SCALE_LINEARX_LINEARY
      DPlot.LegendX       = 0.05
      DPlot.LegendY       = 0.05
      DPlot.NP(1)         = NP
      DPlot.LineType(1)   = LINESTYLE_NONE
      DPlot.SymbolType(1) = 288
      DPlot.Title(1)      = 'Collatz Conjecture' 
      DPlot.YAxis         = 'Number of iterations'

      ret = DPlot_Plot8(DPlot,x,y,
     +   '[Caption("DPLOTLIB Test")][DocMaximize()]'//
     +   '[ClearEditFlag()][SymbolSizeUnits(0)][SymbolSize(1,100)]'//
     +   '[SymbolOpacity(1,10)]'//
     +   '[TextNoteEx("{\bCollatz Conjecture:}\n'//
     +   'Take any natural number {\in}. If {\in} is even, divide '//
     +   'it by 2\n'//
     +   'to get {\in}/2, if {\in} is odd multiply it by 3 and '//
     +   'add 1 to\n'//
     +   'obtain 3{\in} + 1. Repeat the process indefinitely. The\n'//
     +   'conjecture is that no matter what number you start with,\n'//
     +   'you will always eventually reach 1. This example plots\n'//
     +   'the number of iterations required to reach 1 for all\n'//
     +   'numbers between 1 and 100,000 (and also shows off\n'//
     +   'DPlot''s partially transparent symbols feature).",0,0,0,0)]'//
     +   char(0))
      return
      end
