$COMPILE EXE
$RESOURCE "PBTEST3.PBR"
$INCLUDE "WIN32API.INC"
#INCLUDE "CommCtrl.inc"

'===========  [ Program controls identifiers ]  =======================
%DLG_METHOD      = 1000
%DLG_START       = 1001
%DLG_INCREMENT   = 1002
%DLG_MSCALELABEL = 1003
%DLG_MSCALE      = 1004
%DLG_TIMELABEL   = 1005
%DLG_TIME        = 1006
%DLG_MAIN        = 1020

'===========  [ Program GLOBAL declarations ]  ========================

GLOBAL ghInst         AS DWORD         ' Module instance handle.
GLOBAL ghCurrentDlg   AS DWORD         ' Active modeless dlg handle.

GLOBAL TimeStep AS SINGLE
GLOBAL hwndDPlot AS DWORD
GLOBAL hwndDlg AS DWORD
GLOBAL DocNum AS LONG
GLOBAL DrawMethod AS LONG
GLOBAL StartTicks AS LONG
GLOBAL StopTicks AS LONG
GLOBAL Running AS LONG
GLOBAL DPlotWasActive AS LONG

#INCLUDE "DPLOT.INC"

'===========  [ Program ENTRY point ]  ================================
FUNCTION WINMAIN(BYVAL hCurrInstance AS LONG,_
                 BYVAL hPrevInstance AS LONG,_
                 BYVAL lpCmdLine     AS ASCIIZ PTR,_
                 BYVAL nCmdShow      AS LONG) AS LONG

  ' Local variables.
  LOCAL  Msg            AS tagMsg
  LOCAL  wClass         AS WndClass
  LOCAL  hWnd           AS DWORD
  LOCAL  szClassName    AS ASCIIZ * 12

  szClassName = "PBTEST3"
  ghInst = hCurrInstance

  ' A REPLACEMENT for program PrevInstance.
  IF ISFALSE(FindWindow(szClassName, BYVAL %NULL)) THEN
     ' Fill window structure.
     wClass.style         = %CS_HREDRAW OR %CS_VREDRAW
     wClass.lpfnWndProc   = GetProcAddress(GetModuleHandle("USER32"), "DefDlgProcA")
     wClass.cbClsExtra    = 0
     wClass.cbWndExtra    = %DLGWINDOWEXTRA
     wClass.hInstance     = hCurrInstance
     wClass.hIcon         = LoadIcon(%NULL, BYVAL %IDI_APPLICATION)
     wClass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
     wClass.hbrBackground = GetStockObject(%LTGRAY_BRUSH)
     wClass.lpszMenuName  = %NULL
     wClass.lpszClassName = VARPTR(szClassName)

     ' Register the window-class.
     RegisterClass wClass
  ELSE
     ' Bring prev instance to front.
     hWnd = FindWindow(szClassName, BYVAL %NULL)
     IF IsIconic(hWnd) THEN ShowWindow hWnd ,%SW_RESTORE
     SetForegroundWindow hWnd
     ' Exit.
     FUNCTION = 0
     EXIT FUNCTION
  END IF

  ' Create the main window as a MODELESS dialog.
  hWnd = CreateDialogParam(hCurrInstance, BYVAL CLNG(%DLG_MAIN), 0, CODEPTR(MainWndProc), 0)
  ShowWindow hWnd, nCmdShow

  ' Main message loop of program.
  WHILE GetMessage(Msg, %NULL, 0, 0)
      IF ISFALSE(ghCurrentDlg) OR ISFALSE(IsDialogMessage(ghCurrentDlg, Msg)) THEN
         TranslateMessage Msg
         DispatchMessage Msg
      END IF
  WEND

  ' Clean up and assign return value.

  FUNCTION = Msg.wParam

END FUNCTION

'==============================================================================

SUB RealTimeStop()
  DIM ret AS LONG

  KillTimer(hwndDlg,1)
  StopTicks = GetTickCount()
  Running = 0
  ' Redraw the plot and re-enable menu updates
  ret = DPlot_Command(DocNum, "[ViewRedraw()][MenuUpdates(1)]")
  ' Make time output invisible
  ShowWindow(GetDlgItem(hwndDlg,%DLG_TIMELABEL),%SW_HIDE)
  ShowWindow(GetDlgItem(hwndDlg,%DLG_TIME),%SW_HIDE)
  ' Reset button text
  SetDlgItemText(hwndDlg,%DLG_START,"&Start")

END SUB

'==============================================================================

SUB TimerProc(BYVAL hwnd AS DWORD, BYVAL uMsg AS DWORD, BYVAL idEvent AS DWORD, BYVAL dwTime AS DWORD)

  DIM x AS SINGLE
  DIM y1 AS SINGLE
  DIM y2 AS SINGLE
  DIM szExec AS ASCIIZ*256
  DIM ret AS LONG

  x = (dwTime - StartTicks) / 1000.0!
  y1 = SIN(x/4)
  y2 = COS(x/4)

  ' The most efficient method for sending data to DPlot is highly dependent
  ' on how many points are sent at once and how many curves are updated at
  ' once. For this demo it's a wash; they're roughly equivalent. The
  ' advantage of using DPlot_Command with an XY or XYXY command is that
  ' multiple curves may be updated in a single call. The advantage of using
  ' DPlot_AddData is that the data is sent in its native binary format, so
  ' there's no need for translation to character string and back to binary.
  ' For a large number of points this advantage becomes much more important.
  ' DPlot_AddData also allows you to send an arbitrarily large number of
  ' values; DPlot_Command is limited by the maximum command string length
  ' (32768 characters).

  IF DrawMethod THEN
      ret = DPlot_AddData(DocNum, %DATA_XYXY, 1, 1, x, y1)
      IF ret > 0 THEN ret = DPlot_AddData(DocNum, %DATA_XYXY, 1, 2, x, y2)
  ELSE
      szExec = "[SelectCurve(1)][XY(" & STR$(x) & "," & STR$(y1) & ")]" & _
               "[SelectCurve(2)][XY(" & STR$(x) & "," & STR$(y2) & ")]"
      ret = DPlot_Command(DocNum,szExec)
  END IF

  IF ret <= 0 THEN
      RealTimeStop()
  ELSE
      SetDlgItemText(hwndDlg,%DLG_TIME,STR$(x))
  END IF
END SUB

'===============  [ MainWnd dialog window procedure ]  =================
FUNCTION MainWndProc(BYVAL hDlg    AS DWORD,_
                     BYVAL wMsg    AS DWORD,_
                     BYVAL wParam  AS DWORD,_
                     BYVAL lParam  AS DWORD) EXPORT AS LONG

  DIM szText AS ASCIIZ*64
  DIM szExec AS ASCIIZ*256
  DIM cx AS LONG
  DIM cy AS LONG
  DIM rc AS RECT
  DIM rcWork AS RECT

  SELECT CASE wMsg
      CASE %WM_INITDIALOG
          ' Preserve window handle for use in other functions
          hwndDlg = hDlg

          StopTicks = 0

          ' Initial time step = 0.1 seconds
          Running = 0
          SetDlgItemText hDlg,%DLG_INCREMENT,"0.1"

          DrawMethod = 0        ' Use DPlot_Command
          CheckDlgButton hDlg, %DLG_METHOD, DrawMethod

          ' Initial plot window
          SetDlgItemText hDlg, %DLG_MSCALE, "0.,-1.5,60.,1.5"

          ' Make time output invisible while simulation isn't running
          ShowWindow(GetDlgItem(hDlg,%DLG_TIMELABEL),%SW_HIDE)
          ShowWindow(GetDlgItem(hDlg,%DLG_TIME),%SW_HIDE)

          ' Set dialog position to upper left corner of work area and make it
          ' topmost (stay-on-top)
          SystemParametersInfo(%SPI_GETWORKAREA,0,rcWork,0)
          SetWindowPos(hDlg,%HWND_TOPMOST,rcWork.nLeft,rcWork.nTop,0,0,%SWP_NOSIZE)

          ' Start DPlot
          hwndDPlot = DPlot_Start(0,DPlotWasActive)

          ' Move DPlot to upper right corner of screen, sized so that both our form
          ' and DPlot are completely visible
          GetWindowRect(hDlg,rc)
          cx = rcWork.nRight - rcWork.nLeft - rc.nRight
          cy = rcWork.nBottom - rcWork.nTop
          MoveWindow hwndDPlot, rc.nRight, rcWork.nTop, cx, cy, 1

          ' Use MessageBox to report errors
          DPlot_SetErrorMethod(2)

          FUNCTION = %TRUE

      CASE %WM_ACTIVATE
          IF LOWRD(wParam) = 0 THEN  ' Dialog becoming inactive.
              ghCurrentDlg = 0
          ELSE                       ' Dialog becoming active.
              ghCurrentDlg = hDlg
          END IF
          FUNCTION = %TRUE

      CASE %WM_DESTROY
          ' Send a message to terminate message loop.
          PostQuitMessage 0
          FUNCTION = %TRUE

      CASE %WM_COMMAND
          SELECT CASE LOWRD(wParam)
              CASE %DLG_START
                  ' If simulation is already running, stop it
                  IF Running THEN
                      RealTimeStop()
                      FUNCTION = %TRUE
                      EXIT FUNCTION
                  END IF

                  ' With a very short time step, we may end up here twice in rapid succession,
                  ' and user will have a hard time stopping the demo. This gives us a
                  ' governor:
                  IF GetTickCount()-StopTicks < 1000 THEN
                      FUNCTION = %TRUE
                      EXIT FUNCTION
                  END IF

                  ' Get timestep
                  GetDlgItemText hDlg, %DLG_INCREMENT, szText, SIZEOF(szText)
                  TimeStep = VAL(szText)

                  ' Drawing method (see comments in TimerProc)
                  DrawMethod = IsDlgButtonChecked(hDlg, %DLG_METHOD)

                  ' Change button caption to indicate that clicking it again stops the simulation,
                  ' and make the time display visible
                  SetDlgItemText hDlg, %DLG_START, "&Stop"
                  ShowWindow(GetDlgItem(hDlg,%DLG_TIMELABEL),%SW_SHOW)
                  ShowWindow(GetDlgItem(hDlg,%DLG_TIME),%SW_SHOW)

                  ' If DPlot is already running, bring it to the top in case user might have
                  ' minimized it.
                  IF IsWindow(hwndDPlot) THEN
                      IF IsIconic(hwndDPlot) THEN
                          ShowWindow hwndDPlot, %SW_RESTORE
                      ELSE
                          BringWindowToTop hwndDPlot
                      END IF
                  END IF

                  ' Create an empty document window in DPlot and get the document index, which will
                  ' be used for all subsequent DPlot_Command command strings.
                  DocNum = DPlot_Command(0, "[FileNew()]")

                  ' Instruct DPLOT to allocate space for 2 curves, 1 hour's
                  ' worth of data:
                  szExec = "[FileArrays(2," & STR$(INT(3600/TimeStep-0.5)+1) & ")]"
                  DPlot_Command(DocNum,szExec)

                  ' Use thick (.03") lines to push DPlot's performance limits a bit, and
                  ' set a few other plot parameters
                  DPlot_Command(DocNum,"[LineWidth(1,30)][LineWidth(2,30)][LineType(1,1)][LineType(2,1)]" & _
                                       "[Color(1,255,0,0)][Color(2,0,0,255)]")

                  ' Get plot window
                  GetDlgItemText hDlg, %DLG_MSCALE, szText, SIZEOF(szText)
                  szExec = "[ManualScale(" & szText & ")][AutoSize()]"
                  DPlot_Command(DocNum,szExec)

                  ' Set labels
                  DPlot_Command(DocNum,"[XAxisLabel(" & CHR$(34) & "Time (seconds)" & CHR$(34) & ")]" & _
                    "[LegendParams(0.5,10,1,1,.05,.05)][FontPoints(1,8)][FontPoints(2,12)][FontPoints(4,12)][FontPoints(6,10)]" & _
                    "[Legend(1," & CHR$(34) & "SIN(t/4)" & CHR$(34) & ")]" & _
                    "[Legend(2," & CHR$(34) & "COS(t/4)" & CHR$(34) & ")]" & _
                    "[Caption(" & CHR$(34) & "C Demo" & CHR$(34) & ")]")

                  IF DrawMethod THEN
                      DPlot_Command(DocNum,"[Title1(" & CHR$(34) & "RealTime Demo using DPlot_AddData" & CHR$(34) & ")]")
                  ELSE
                      DPlot_Command(DocNum,"[Title1(" & CHR$(34) & "RealTime Demo using DPlot_Command" & CHR$(34) & ")]")
                  END IF

                  ' Tell DPlot not to update its menu (which normally would be done every
                  ' time [SelectCurve()] is used). This isn't strictly necessary, but
                  ' avoids annoying flashing from the menu and toolbar being continually
                  ' redrawn, and is a bit more efficient since, for the purposes of this
                  ' demo, the menu doesn't really need to be updated anyway. HOWEVER,
                  ' this setting is persistent. If you turn off menu updates then you
                  ' must eventually turn them back on. In the case of this example this
                  ' is done when the user stops the simulation and RealTimeStop is called.
                  DPlot_Command(DocNum, "[MenuUpdates(0)]")

                  ' Get start time from API function GetTickCount
                  StartTicks = GetTickCount()

                  ' Start the timer. See TimerProc for sending our values to DPlot
                  SetTimer hDlg, 1, INT(TimeStep*1000), CODEPTR(TimerProc)

                  Running = 1

                  FUNCTION = %TRUE

              CASE ELSE
                  FUNCTION = %FALSE

          END SELECT

      CASE %WM_SYSCOMMAND
          SELECT CASE wParam
              CASE %SC_CLOSE
                  IF Running THEN RealTimeStop()
                  DestroyWindow hDlg
                  FUNCTION = %TRUE
              CASE ELSE
                  FUNCTION = %FALSE
          END SELECT

      CASE ELSE
          FUNCTION = %FALSE

  END SELECT

END FUNCTION
